@@ -218,7 +218,7 @@ gst_alsa_sink_check_event (GstAlsaSink * sink, gint pad_nr)
         break;
       case GST_EVENT_DISCONTINUOUS:
       {
-        GstClockTime value = GST_CLOCK_TIME_NONE, delay;
+        gint64 value = (gint64) GST_CLOCK_TIME_NONE, delay;
 
         /* only the first pad may seek */
         if (pad_nr != 0) {
@@ -291,7 +291,7 @@ gst_alsa_sink_check_event (GstAlsaSink * sink, gint pad_nr)
         break;
     }
     GST_LOG_OBJECT (sink, "unreffing event %p of type %d with refcount %d",
-        event, GST_EVENT_TYPE (event), GST_DATA_REFCOUNT (event));
+        event, GST_EVENT_TYPE (event), GST_DATA_REFCOUNT_VALUE (event));
     gst_event_unref (event);
   } else {
     /* the element at the top of the chain did not emit an event. */
@@ -329,7 +329,9 @@ gst_alsa_sink_mmap (GstAlsa * this, snd_pcm_sframes_t * avail)
     }
   }
 
-  if ((err = snd_pcm_mmap_begin (this->handle, &dst, &offset, avail)) < 0) {
+  err = snd_pcm_mmap_begin (this->handle, &dst, &offset,
+      (snd_pcm_uframes_t *) avail);
+  if (err < 0) {
     GST_ERROR_OBJECT (this, "mmap failed: %s", snd_strerror (err));
     goto out;
   }
