@@ -971,7 +971,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
   if (metadata_length == 0)
     return;
 
-  data = g_new (gchar, metadata_length + 1);
+  data = g_new (guchar, metadata_length + 1);
   pos = data;
 
   while (pos - data < metadata_length) {
@@ -987,7 +987,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
   }
 
   data[metadata_length] = 0;
-  tags = g_strsplit (data, "';", 0);
+  tags = g_strsplit ((gchar *) data, "';", 0);
 
   for (i = 0; tags[i]; i++) {
     if (!g_ascii_strncasecmp (tags[i], "StreamTitle=", 12)) {
@@ -1145,7 +1145,17 @@ gst_gnomevfssrc_open_file (GstGnomeVFSSrc * src)
   gst_gnomevfssrc_push_callbacks (src);
 
   if (src->uri != NULL) {
-    result = gnome_vfs_open_uri (&(src->handle), src->uri, GNOME_VFS_OPEN_READ);
+    result = gnome_vfs_open_uri (&(src->handle), src->uri,
+        GNOME_VFS_OPEN_READ | GNOME_VFS_OPEN_RANDOM);
+
+    if (result == GNOME_VFS_ERROR_INVALID_OPEN_MODE ||
+        result == GNOME_VFS_ERROR_NOT_SUPPORTED) {
+
+      /* seeking might not be supported, try without it */
+      result = gnome_vfs_open_uri (&(src->handle), src->uri,
+          GNOME_VFS_OPEN_READ);
+    }
+
     if (result != GNOME_VFS_OK) {
       gchar *filename = gnome_vfs_uri_to_string (src->uri,
           GNOME_VFS_URI_HIDE_PASSWORD);
