@@ -83,6 +83,7 @@ struct _GstMad
   gint index_id;
 
   gboolean check_for_xing;
+  gboolean xing_found;
 };
 
 struct _GstMadClass
@@ -365,6 +366,7 @@ gst_mad_init (GstMad * mad)
   mad->half = FALSE;
   mad->ignore_crc = TRUE;
   mad->check_for_xing = TRUE;
+  mad->xing_found = FALSE;
   GST_FLAG_SET (mad, GST_ELEMENT_EVENT_AWARE);
 }
 
@@ -847,7 +849,7 @@ gst_mad_update_info (GstMad * mad)
 G_STMT_START{							\
   if (mad->header.h1 != header->h1 || mad->new_header) {	\
     mad->header.h1 = header->h1;				\
-     changed = TRUE;						\
+    changed = TRUE;						\
   };								\
 } G_STMT_END
 
@@ -865,10 +867,7 @@ G_STMT_START{							\
   CHECK_HEADER (layer, "layer");
   CHECK_HEADER (mode, "mode");
   CHECK_HEADER (emphasis, "emphasis");
-
-  if (header->bitrate != mad->header.bitrate || mad->new_header) {
-    mad->header.bitrate = header->bitrate;
-  }
+  mad->header.bitrate = header->bitrate;
   mad->new_header = FALSE;
 
   if (changed) {
@@ -887,11 +886,13 @@ G_STMT_START{							\
         GST_TAG_LAYER, mad->header.layer,
         GST_TAG_MODE, mode->value_nick,
         GST_TAG_EMPHASIS, emphasis->value_nick, NULL);
-    gst_element_found_tags (GST_ELEMENT (mad), list);
-    gst_tag_list_free (list);
+    if (!mad->xing_found) {
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, mad->header.bitrate, NULL);
+    }
+    gst_element_found_tags_for_pad (GST_ELEMENT (mad), mad->srcpad, 0, list);
   }
 #undef CHECK_HEADER
-
 }
 
 static void
@@ -917,7 +918,7 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
         if (gst_formats_contains (formats, GST_EVENT_DISCONT_OFFSET (event,
                     i).format)) {
           gint64 value = GST_EVENT_DISCONT_OFFSET (event, i).value;
-          gint64 time;
+          gint64 time, total_samples;
           GstFormat format;
           GstEvent *discont;
 
@@ -939,9 +940,10 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
            * that this doesn't happen anywhere so far). */
           format = GST_FORMAT_DEFAULT;
           if (!gst_pad_convert (mad->srcpad,
-                  GST_FORMAT_TIME, time, &format, &mad->total_samples)) {
+                  GST_FORMAT_TIME, time, &format, &total_samples)) {
             continue;
           }
+          mad->total_samples = total_samples;
 
           if (GST_PAD_IS_USABLE (mad->srcpad)) {
             discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
@@ -1250,7 +1252,7 @@ gst_mad_chain (GstPad * pad, GstData * _data)
   }
 
   /* handle data */
-  data = GST_BUFFER_DATA (buffer);
+  data = (gchar *) GST_BUFFER_DATA (buffer);
   size = GST_BUFFER_SIZE (buffer);
 
   /* process the incoming buffer in chunks of maximum MAD_BUFFER_MDLEN bytes;
@@ -1396,23 +1398,20 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       }
 
       if (mad->check_for_xing) {
-        int bitrate, time;
+        int bitrate = 0, time = 0;
         GstTagList *list;
         int frame_len = mad->stream.next_frame - mad->stream.this_frame;
 
         /* Assume Xing headers can only be the first frame in a mp3 file */
         if (mpg123_parse_xing_header (&mad->frame.header,
                 mad->stream.this_frame, frame_len, &bitrate, &time)) {
+          mad->xing_found = TRUE;
           list = gst_tag_list_new ();
           gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
               GST_TAG_DURATION, (gint64) time * 1000 * 1000 * 1000,
               GST_TAG_BITRATE, bitrate, NULL);
-          gst_element_found_tags (GST_ELEMENT (mad), list);
-          if (GST_PAD_IS_USABLE (mad->srcpad)) {
-            gst_pad_push (mad->srcpad, GST_DATA (gst_event_new_tag (list)));
-          } else {
-            gst_tag_list_free (list);
-          }
+          gst_element_found_tags_for_pad (GST_ELEMENT (mad), mad->srcpad, 0,
+              list);
         }
 
         mad->check_for_xing = FALSE;
@@ -1436,9 +1435,11 @@ gst_mad_chain (GstPad * pad, GstData * _data)
         /* if we have a pending timestamp, we can use it now to calculate the sample offset */
         if (GST_CLOCK_TIME_IS_VALID (mad->last_ts)) {
           GstFormat format = GST_FORMAT_DEFAULT;
+          gint64 total_samples;
 
           gst_pad_convert (mad->srcpad, GST_FORMAT_TIME, mad->last_ts, &format,
-              &mad->total_samples);
+              &total_samples);
+          mad->total_samples = total_samples;
           mad->last_ts = GST_CLOCK_TIME_NONE;
         }
         time_offset =
@@ -1491,6 +1492,8 @@ gst_mad_chain (GstPad * pad, GstData * _data)
           }
         }
 
+        GST_DEBUG ("Pushing buf with ts %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuffer)));
         gst_pad_push (mad->srcpad, GST_DATA (outbuffer));
       }
 
