@@ -66,7 +66,7 @@ static gboolean gst_asf_demux_add_video_stream (GstASFDemux * asf_demux,
 static gboolean gst_asf_demux_add_audio_stream (GstASFDemux * asf_demux,
     asf_stream_audio * audio, guint16 id);
 static gboolean gst_asf_demux_setup_pad (GstASFDemux * asf_demux,
-    GstPad * src_pad, GstCaps * caps, guint16 id);
+    GstPad * src_pad, GstCaps * caps, guint16 id, gboolean video);
 
 static GstElementStateReturn gst_asf_demux_change_state (GstElement * element);
 
@@ -145,8 +145,6 @@ gst_asf_demux_class_init (GstASFDemuxClass * klass)
 static void
 gst_asf_demux_init (GstASFDemux * asf_demux)
 {
-  guint i;
-
   asf_demux->sinkpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&gst_asf_demux_sink_template), "sink");
@@ -155,16 +153,6 @@ gst_asf_demux_init (GstASFDemux * asf_demux)
   gst_element_set_loop_function (GST_ELEMENT (asf_demux), gst_asf_demux_loop);
 
   /* We should zero everything to be on the safe side */
-  for (i = 0; i < GST_ASF_DEMUX_NUM_VIDEO_PADS; i++) {
-    asf_demux->video_pad[i] = NULL;
-    asf_demux->video_PTS[i] = 0;
-  }
-
-  for (i = 0; i < GST_ASF_DEMUX_NUM_AUDIO_PADS; i++) {
-    asf_demux->audio_pad[i] = NULL;
-    asf_demux->audio_PTS[i] = 0;
-  }
-
   asf_demux->num_audio_streams = 0;
   asf_demux->num_video_streams = 0;
   asf_demux->num_streams = 0;
@@ -599,7 +587,7 @@ gst_asf_demux_process_comment (GstASFDemux * asf_demux, guint64 * obj_size)
         goto fail;
 
       /* convert to UTF-8 */
-      utf8_comments[i] = g_convert (data, lengths[i],
+      utf8_comments[i] = g_convert ((char *) data, lengths[i],
           "UTF-8", "UTF-16LE", &in, &out, NULL);
 
       gst_bytestream_flush_fast (bs, lengths[i]);
@@ -669,13 +657,13 @@ IsVBR
 
 */
 
-  const guchar *tags[] = { GST_TAG_GENRE, GST_TAG_ALBUM, GST_TAG_ARTIST, GST_TAG_TRACK_NUMBER, GST_TAG_DATE, NULL };    // GST_TAG_COMPOSER
-  const guchar *tags_label[] = { "WM/Genre", "WM/AlbumTitle", "WM/AlbumArtist", "WM/TrackNumber", "WM/Year", NULL };    // "WM/Composer"
+  const gchar *tags[] = { GST_TAG_GENRE, GST_TAG_ALBUM, GST_TAG_ARTIST, GST_TAG_TRACK_NUMBER, GST_TAG_DATE, NULL };     // GST_TAG_COMPOSER
+  const gchar *tags_label[] = { "WM/Genre", "WM/AlbumTitle", "WM/AlbumArtist", "WM/TrackNumber", "WM/Year", NULL };     // "WM/Composer"
 
   GstTagList *taglist;
-  GValue tag_value = { 0 };
+  GValue tag_value = { 0, };
   gboolean have_tags = FALSE;
-  guint8 *name = NULL;
+  char *name = NULL;
 
   GST_INFO ("Object is an extended content description.");
 
@@ -690,7 +678,8 @@ IsVBR
     guint16 datatype;
     guint16 value_length;
     guint8 *tmpname;
-    guint8 *tmpvalue, *value;
+    guint8 *tmpvalue;
+    char *value;
 
     /* Descriptor Name Length */
     if (!_read_uint16 (asf_demux, &name_length))
@@ -744,8 +733,20 @@ IsVBR
 
             /* get rid of tags with empty value */
             if (strlen (value)) {
-              g_value_init (&tag_value, G_TYPE_STRING);
-              g_value_set_string (&tag_value, value);
+              if (strcmp (tags[tag], GST_TAG_DATE) == 0) {
+                guint year = atoi (value);
+
+                if (year > 0) {
+                  GDate *date = g_date_new_dmy (1, 1, year);
+
+                  g_value_init (&tag_value, G_TYPE_INT);
+                  g_value_set_int (&tag_value, g_date_get_julian (date));
+                  g_date_free (date);
+                }
+              } else {
+                g_value_init (&tag_value, G_TYPE_STRING);
+                g_value_set_string (&tag_value, value);
+              }
             }
           }
 
@@ -1009,10 +1010,14 @@ gst_asf_demux_handle_data (GstASFDemux * asf_demux)
 
   GST_INFO ("Process packet");
 
-  if (asf_demux->packet++ >= asf_demux->num_packets) {
+  if (asf_demux->num_packets > 0 &&
+      asf_demux->packet++ >= asf_demux->num_packets) {
     GstEvent *event;
     guint32 remaining;
 
+    GST_LOG ("Number of packets (%d > %d) reached, EOS",
+        asf_demux->packet - 1, asf_demux->num_packets);
+
     gst_bytestream_flush (asf_demux->bs, 0xFFFFFF);
     gst_bytestream_get_status (asf_demux->bs, &remaining, &event);
     if (!event || GST_EVENT_TYPE (event) != GST_EVENT_EOS)
@@ -1359,10 +1364,13 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
   GstBuffer *buffer;
 
   if (!(stream =
-          gst_asf_demux_get_stream (asf_demux, segment_info->stream_number)))
+          gst_asf_demux_get_stream (asf_demux, segment_info->stream_number))) {
+    GST_DEBUG ("Invalid stream number %d", segment_info->stream_number);
     goto done;
+  }
 
-  GST_DEBUG ("Processing chunk of size %u (fo = %d)", segment_info->chunk_size,
+  GST_DEBUG ("Processing %s chunk of size %u (fo = %d)",
+      gst_pad_get_name (stream->pad), segment_info->chunk_size,
       stream->frag_offset);
 
   if (segment_info->frag_offset == 0) {
@@ -1444,8 +1452,12 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
       GST_DEBUG ("New buffer is at: %p size: %u",
           GST_BUFFER_DATA (stream->payload), GST_BUFFER_SIZE (stream->payload));
 
+      if (asf_demux->pts >= stream->last_pts ||
+          !GST_CLOCK_TIME_IS_VALID (stream->last_pts))
+        stream->last_pts = asf_demux->pts;
+
       GST_BUFFER_TIMESTAMP (stream->payload) =
-          (GST_SECOND / 1000) * asf_demux->pts;
+          (GST_SECOND / 1000) * stream->last_pts;
 
       /*!!! Should handle flush events here? */
       GST_DEBUG ("Sending stream %d of size %d", stream->id,
@@ -1465,8 +1477,27 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
 
         asf_demux->seek_discont = FALSE;
       }
+      if (!stream->fps_known) {
+        if (!stream->cache) {
+          stream->cache = stream->payload;
+        } else {
+          gdouble fps = (gdouble) GST_SECOND /
+              (GST_BUFFER_TIMESTAMP (stream->payload) -
+              GST_BUFFER_TIMESTAMP (stream->cache));
 
-      gst_pad_push (stream->pad, GST_DATA (stream->payload));
+          stream->fps_known = TRUE;
+          gst_caps_set_simple (stream->caps,
+              "framerate", G_TYPE_DOUBLE, fps, NULL);
+          GST_DEBUG ("Set-up stream with fps %lf", fps);
+          gst_pad_set_explicit_caps (stream->pad, stream->caps);
+
+          gst_pad_push (stream->pad, GST_DATA (stream->cache));
+          gst_pad_push (stream->pad, GST_DATA (stream->payload));
+          stream->cache = NULL;
+        }
+      } else {
+        gst_pad_push (stream->pad, GST_DATA (stream->payload));
+      }
     } else {
       gst_buffer_unref (stream->payload);
       stream->payload = NULL;
@@ -1672,7 +1703,6 @@ static GstElementStateReturn
 gst_asf_demux_change_state (GstElement * element)
 {
   GstASFDemux *asf_demux = GST_ASF_DEMUX (element);
-  gint i;
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_READY_TO_PAUSED:
@@ -1681,12 +1711,6 @@ gst_asf_demux_change_state (GstElement * element)
       break;
     case GST_STATE_PAUSED_TO_READY:
       gst_bytestream_destroy (asf_demux->bs);
-      for (i = 0; i < GST_ASF_DEMUX_NUM_VIDEO_PADS; i++) {
-        asf_demux->video_PTS[i] = 0;
-      }
-      for (i = 0; i < GST_ASF_DEMUX_NUM_AUDIO_PADS; i++) {
-        asf_demux->audio_PTS[i] = 0;
-      }
       if (asf_demux->taglist) {
         gst_tag_list_free (asf_demux->taglist);
         asf_demux->taglist = NULL;
@@ -1774,7 +1798,7 @@ gst_asf_demux_add_audio_stream (GstASFDemux * asf_demux,
 
   asf_demux->num_audio_streams++;
 
-  if (!gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id))
+  if (!gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id, FALSE))
     return FALSE;
 
   gst_pad_push (src_pad, GST_DATA (gst_event_new_tag (list)));
@@ -1810,7 +1834,6 @@ gst_asf_demux_add_video_stream (GstASFDemux * asf_demux,
   /* yes, asf_stream_video_format and gst_riff_strf_vids are the same */
   caps = gst_riff_create_video_caps_with_data (video->tag, NULL,
       (gst_riff_strf_vids *) video, extradata, NULL, &codec_name);
-  gst_caps_set_simple (caps, "framerate", G_TYPE_DOUBLE, 25.0, NULL);
   gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
       codec_name, NULL);
   gst_element_found_tags (GST_ELEMENT (asf_demux), list);
@@ -1819,10 +1842,11 @@ gst_asf_demux_add_video_stream (GstASFDemux * asf_demux,
     gst_buffer_unref (extradata);
   GST_INFO ("Adding video stream %u codec " GST_FOURCC_FORMAT " (0x%08x)",
       asf_demux->num_video_streams, GST_FOURCC_ARGS (video->tag), video->tag);
+  gst_caps_set_simple (caps, "framerate", G_TYPE_DOUBLE, 25.0, NULL);
 
   asf_demux->num_video_streams++;
 
-  if (!gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id))
+  if (!gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id, TRUE))
     return FALSE;
 
   gst_pad_push (src_pad, GST_DATA (gst_event_new_tag (list)));
@@ -1833,7 +1857,7 @@ gst_asf_demux_add_video_stream (GstASFDemux * asf_demux,
 
 static gboolean
 gst_asf_demux_setup_pad (GstASFDemux * asf_demux,
-    GstPad * src_pad, GstCaps * caps, guint16 id)
+    GstPad * src_pad, GstCaps * caps, guint16 id, gboolean is_video)
 {
   asf_stream_context *stream;
 
@@ -1846,11 +1870,14 @@ gst_asf_demux_setup_pad (GstASFDemux * asf_demux,
   gst_pad_set_query_function (src_pad, gst_asf_demux_handle_src_query);
 
   stream = &asf_demux->stream[asf_demux->num_streams];
+  stream->caps = caps;
   stream->pad = src_pad;
   stream->id = id;
   stream->frag_offset = 0;
   stream->sequence = 0;
   stream->delay = 0LL;
+  stream->last_pts = GST_CLOCK_TIME_NONE;
+  stream->fps_known = !is_video;        /* bit hacky for audio */
 
   gst_pad_set_element_private (src_pad, stream);
 
