@@ -26,6 +26,12 @@
 
 #include "gstdvddemux.h"
 
+/* 
+ * Move PTM discont back by 0.3 seconds to allow for strange audio
+ * timestamps when audio crosses a VOBU 
+ */
+#define PTM_DISCONT_ADJUST (0.3 * GST_SECOND)
+#define INITIAL_END_PTM (-1)
 
 GST_DEBUG_CATEGORY_STATIC (gstdvddemux_debug);
 #define GST_CAT_DEFAULT (gstdvddemux_debug)
@@ -77,20 +83,20 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
   GST_STATIC_CAPS ( \
     "audio/mpeg, " \
       "mpegversion = (int) 1;" \
-    "audio/x-raw-int, " \
-      "endianness = (int) BIG_ENDIAN, " \
-      "signed = (boolean) TRUE, " \
+    "audio/x-dvd-lpcm, " \
       "width = (int) { 16, 20, 24 }, " \
-      "depth = (int) { 16, 20, 24 }, " \
       "rate = (int) { 48000, 96000 }, " \
-      "channels = (int) [ 1, 8 ];" \
-    "audio/x-ac3" \
+      "channels = (int) [ 1, 8 ], " \
+      "dynamic_range = (int) [ 0, 255 ], " \
+      "emphasis = (boolean) { FALSE, TRUE }, " \
+      "mute = (boolean) { FALSE, TRUE }; " \
+    "audio/x-ac3;" \
+    "audio/x-dts" \
   )
 
 #define SUBPICTURE_CAPS \
   GST_STATIC_CAPS ("video/x-dvd-subpicture")
 
-
 static GstStaticPadTemplate cur_video_template =
 GST_STATIC_PAD_TEMPLATE ("current_video",
     GST_PAD_SRC,
@@ -121,7 +127,6 @@ GST_STATIC_PAD_TEMPLATE ("current_subpicture",
     GST_PAD_ALWAYS,
     SUBPICTURE_CAPS);
 
-
 static void gst_dvd_demux_class_init (GstDVDDemuxClass * klass);
 static void gst_dvd_demux_base_init (GstDVDDemuxClass * klass);
 static void gst_dvd_demux_init (GstDVDDemux * dvd_demux);
@@ -129,10 +134,12 @@ static void gst_dvd_demux_init (GstDVDDemux * dvd_demux);
 static void gst_dvd_demux_send_data (GstMPEGParse * mpeg_parse,
     GstData * data, GstClockTime time);
 
-static void gst_dvd_demux_send_discont
-    (GstMPEGParse * mpeg_parse, GstClockTime time);
+static void gst_dvd_demux_handle_discont
+    (GstMPEGParse * mpeg_parse, GstEvent * event);
 static gboolean gst_dvd_demux_handle_dvd_event
     (GstDVDDemux * dvd_demux, GstEvent * event);
+static void gst_dvd_demux_send_event
+    (GstMPEGParse * mpeg_parse, GstEvent * event, GstClockTime time);
 
 static GstMPEGStream *gst_dvd_demux_get_video_stream
     (GstMPEGDemux * mpeg_demux,
@@ -159,6 +166,13 @@ static void gst_dvd_demux_set_cur_audio
 static void gst_dvd_demux_set_cur_subpicture
     (GstDVDDemux * dvd_demux, gint stream_nr);
 
+static void gst_dvd_demux_reset (GstDVDDemux * dvd_demux);
+static void gst_dvd_demux_synchronise_pads (GstMPEGDemux * mpeg_demux,
+    GstClockTime threshold, GstClockTime new_ts);
+static void gst_dvd_demux_sync_stream_to_time (GstMPEGDemux * mpeg_demux,
+    GstMPEGStream * stream, GstClockTime last_ts);
+
+static GstElementStateReturn gst_dvd_demux_change_state (GstElement * element);
 
 static GstMPEGDemuxClass *parent_class = NULL;
 
@@ -221,6 +235,7 @@ gst_dvd_demux_base_init (GstDVDDemuxClass * klass)
 
   gst_element_class_add_pad_template (element_class, klass->cur_video_template);
   gst_element_class_add_pad_template (element_class, klass->cur_audio_template);
+
   gst_element_class_add_pad_template (element_class,
       klass->subpicture_template);
   gst_element_class_add_pad_template (element_class,
@@ -243,12 +258,17 @@ gst_dvd_demux_class_init (GstDVDDemuxClass * klass)
   mpeg_parse_class = (GstMPEGParseClass *) klass;
   mpeg_demux_class = (GstMPEGDemuxClass *) klass;
 
-  mpeg_parse_class->send_discont = gst_dvd_demux_send_discont;
+  gstelement_class->change_state = gst_dvd_demux_change_state;
+
+  mpeg_parse_class->handle_discont = gst_dvd_demux_handle_discont;
+  mpeg_parse_class->send_event = gst_dvd_demux_send_event;
 
   mpeg_demux_class->get_audio_stream = gst_dvd_demux_get_audio_stream;
   mpeg_demux_class->get_video_stream = gst_dvd_demux_get_video_stream;
   mpeg_demux_class->send_subbuffer = gst_dvd_demux_send_subbuffer;
   mpeg_demux_class->process_private = gst_dvd_demux_process_private;
+  mpeg_demux_class->synchronise_pads = gst_dvd_demux_synchronise_pads;
+  mpeg_demux_class->sync_stream_to_time = gst_dvd_demux_sync_stream_to_time;
 
   klass->get_subpicture_stream = gst_dvd_demux_get_subpicture_stream;
 }
@@ -281,34 +301,14 @@ gst_dvd_demux_init (GstDVDDemux * dvd_demux)
   dvd_demux->cur_audio_nr = 0;
   dvd_demux->cur_subpicture_nr = 0;
 
-  /* Start the timestamp sequence in 0. */
-  dvd_demux->last_end_ptm = 0;
-
-  /* (Ronald) so, this was disabled. I'm enabling (default) it again.
-   * Timestamp adjustment is fairly evil, we would ideally use discont
-   * events instead. However, our current clocking has a pretty serious
-   * race condition: imagine that $pipeline is at time 30sec and $audio
-   * receives a discont to 0sec. Video processes its last buffer and
-   * calls _wait() on $timestamp, which is 30s - so we wait (hang) 30sec.
-   * This is unacceptable, obviously, and timestamp adjustment, no matter
-   * how evil, solves this.
-   * Before disabling this again, tripple check that al .vob files on our
-   * websites /media/ directory work fine, especially bullet.vob and
-   * barrage.vob.
-   */
-#if 0
-  /* Try to prevent the mpegparse infrastructure from doing timestamp
-     adjustment. */
-  mpeg_parse->do_adjust = FALSE;
-  mpeg_parse->adjust = 0;
-#endif
-
-  dvd_demux->just_flushed = FALSE;
+  dvd_demux->last_end_ptm = INITIAL_END_PTM;
   dvd_demux->discont_time = GST_CLOCK_TIME_NONE;
 
   for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
     dvd_demux->subpicture_stream[i] = NULL;
   }
+
+  dvd_demux->langcodes = NULL;
 }
 
 
@@ -327,17 +327,14 @@ gst_dvd_demux_send_data (GstMPEGParse * mpeg_parse, GstData * data,
       case GST_EVENT_ANY:
         gst_dvd_demux_handle_dvd_event (dvd_demux, event);
         break;
-
+      case GST_EVENT_FILLER:
+      case GST_EVENT_DISCONTINUOUS:
       case GST_EVENT_FLUSH:
-        GST_DEBUG_OBJECT (dvd_demux, "flush received");
-
-        dvd_demux->just_flushed = TRUE;
-
-        /* Propagate the event normally. */
-        gst_pad_event_default (mpeg_parse->sinkpad, event);
+        PARSE_CLASS (dvd_demux)->send_event (mpeg_parse, event,
+            GST_CLOCK_TIME_NONE);
         break;
-
       default:
+        /* Propagate the event normally. */
         gst_pad_event_default (mpeg_parse->sinkpad, event);
         break;
     }
@@ -391,88 +388,126 @@ gst_dvd_demux_handle_dvd_event (GstDVDDemux * dvd_demux, GstEvent * event)
   }
 
   else if (strcmp (event_type, "dvd-nav-packet") == 0) {
+    GstClockTimeDiff old_adjust = mpeg_demux->adjust;
     GstStructure *structure = event->event_data.structure.structure;
     GstClockTime start_ptm =
         g_value_get_uint64 (gst_structure_get_value (structure, "start_ptm"));
     GstClockTime end_ptm =
         g_value_get_uint64 (gst_structure_get_value (structure, "end_ptm"));
+    GstClockTime cell_start =
+        g_value_get_uint64 (gst_structure_get_value (structure, "cell_start"));
+    GstClockTime pg_start =
+        g_value_get_uint64 (gst_structure_get_value (structure, "pg_start"));
 
     if (start_ptm != dvd_demux->last_end_ptm) {
-      /* Set the adjust value to gap the discontinuity. */
-      mpeg_demux->adjust += GST_CLOCK_DIFF (dvd_demux->last_end_ptm, start_ptm);
 
       GST_DEBUG_OBJECT (dvd_demux,
-          "PTM sequence discontinuity: from %0.3fs to "
-          "%0.3fs, new adjust %0.3fs",
+          "PTM sequence discontinuity: from %0.3fs to %0.3fs, cell_start %0.3fs, pg_start %0.3fs",
           (double) dvd_demux->last_end_ptm / GST_SECOND,
           (double) start_ptm / GST_SECOND,
-          (double) mpeg_demux->adjust / GST_SECOND);
+          (double) cell_start / GST_SECOND, (double) pg_start / GST_SECOND);
+
+      if (pg_start > start_ptm)
+        mpeg_demux->adjust = pg_start - start_ptm;
+      else
+        mpeg_demux->adjust = 0;
+
+      /* Try to prevent the mpegparse infrastructure from doing timestamp
+         adjustment, and enable synchronising filler events. */
+      mpeg_parse->use_adjust = FALSE;
+      mpeg_parse->adjust = 0;
+
+      /* Keep video/audio/subtitle pads within 1/2 sec of the SCR */
+      mpeg_demux->max_gap = 0.5 * GST_SECOND;
+      mpeg_demux->max_gap_tolerance = 0.05 * GST_SECOND;
     }
-    dvd_demux->last_end_ptm = end_ptm;
+
+    /* Send a discont after a seek, or if PTM wrapping causes too large a gap */
+    if (mpeg_demux->just_flushed ||
+        ABS (GST_CLOCK_DIFF (dvd_demux->last_end_ptm + old_adjust,
+                start_ptm + mpeg_demux->adjust)) > PTM_DISCONT_ADJUST) {
 
-    if (dvd_demux->just_flushed) {
       /* The pipeline was just flushed, schedule a discontinuity with
          the next sequence time. We don't do it here to reduce the
          time gap between the discontinuity and the subsequent data
          blocks. */
-      dvd_demux->discont_time = start_ptm + mpeg_demux->adjust;
-      dvd_demux->just_flushed = FALSE;
+      if (start_ptm > PTM_DISCONT_ADJUST)
+        dvd_demux->discont_time = start_ptm - PTM_DISCONT_ADJUST;
+      else
+        dvd_demux->discont_time = 0;
+
+      GST_DEBUG_OBJECT (dvd_demux,
+          "Set mpeg discont time to %" G_GINT64_FORMAT ", adjust %"
+          G_GINT64_FORMAT, dvd_demux->discont_time, mpeg_demux->adjust);
     }
 
+    dvd_demux->last_end_ptm = end_ptm;
+
     gst_event_unref (event);
-  }
+  } else if (!strcmp (event_type, "dvd-lang-codes")) {
+    if (dvd_demux->langcodes)
+      gst_event_unref (dvd_demux->langcodes);
+    dvd_demux->langcodes = event;
+  } else {
+    GST_DEBUG_OBJECT (dvd_demux, "dvddemux Forwarding DVD event %s to all pads",
+        event_type);
 
-  else {
-    if (GST_EVENT_TIMESTAMP (event) != GST_CLOCK_TIME_NONE) {
-      GST_EVENT_TIMESTAMP (event) += mpeg_demux->adjust;
-    }
-    gst_pad_event_default (mpeg_parse->sinkpad, event);
+    PARSE_CLASS (dvd_demux)->send_event (mpeg_parse, event,
+        GST_CLOCK_TIME_NONE);
   }
 
   return TRUE;
 }
 
-
 static void
-gst_dvd_demux_send_discont (GstMPEGParse * mpeg_parse, GstClockTime time)
+gst_dvd_demux_send_event (GstMPEGParse * mpeg_parse, GstEvent * event,
+    GstClockTime ts)
 {
   GstDVDDemux *dvd_demux = GST_DVD_DEMUX (mpeg_parse);
-  GstEvent *discont;
   gint i;
 
-  GST_DEBUG_OBJECT (dvd_demux, "sending discontinuity: %0.3fs",
-      (double) time / GST_SECOND);
-
-  GST_MPEG_PARSE_CLASS (parent_class)->send_discont (mpeg_parse, time);
-
   for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
-    if (dvd_demux->subpicture_stream[i] &&
-        GST_PAD_IS_USABLE (dvd_demux->subpicture_stream[i]->pad)) {
-      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
-          time, NULL);
-
-      gst_pad_push (dvd_demux->subpicture_stream[i]->pad, GST_DATA (discont));
+    if (dvd_demux->subpicture_stream[i]) {
+      if (GST_PAD_IS_USABLE (dvd_demux->subpicture_stream[i]->pad)) {
+        gst_event_ref (event);
+        gst_pad_push (dvd_demux->subpicture_stream[i]->pad, GST_DATA (event));
+      }
+      if (GST_CLOCK_TIME_IS_VALID (ts))
+        dvd_demux->subpicture_stream[i]->cur_ts = ts;
     }
   }
 
   /* Distribute the event to the "current" pads. */
   if (GST_PAD_IS_USABLE (dvd_demux->cur_video)) {
-    discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
-
-    gst_pad_push (dvd_demux->cur_video, GST_DATA (discont));
+    gst_event_ref (event);
+    gst_pad_push (dvd_demux->cur_video, GST_DATA (event));
   }
 
   if (GST_PAD_IS_USABLE (dvd_demux->cur_audio)) {
-    discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
-
-    gst_pad_push (dvd_demux->cur_audio, GST_DATA (discont));
+    gst_event_ref (event);
+    gst_pad_push (dvd_demux->cur_audio, GST_DATA (event));
   }
 
   if (GST_PAD_IS_USABLE (dvd_demux->cur_subpicture)) {
-    discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
-
-    gst_pad_push (dvd_demux->cur_subpicture, GST_DATA (discont));
+    gst_event_ref (event);
+    gst_pad_push (dvd_demux->cur_subpicture, GST_DATA (event));
   }
+
+  GST_MPEG_PARSE_CLASS (parent_class)->send_event (mpeg_parse, event, ts);
+}
+
+static void
+gst_dvd_demux_handle_discont (GstMPEGParse * mpeg_parse, GstEvent * event)
+{
+  GstDVDDemux *dvd_demux = GST_DVD_DEMUX (mpeg_parse);
+
+  if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
+    gst_dvd_demux_reset (dvd_demux);
+  }
+
+  /* let parent handle and forward discont */
+  if (GST_MPEG_PARSE_CLASS (parent_class)->handle_discont != NULL)
+    GST_MPEG_PARSE_CLASS (parent_class)->handle_discont (mpeg_parse, event);
 }
 
 static GstMPEGStream *
@@ -500,6 +535,7 @@ gst_dvd_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
     gst_caps_free (caps);
   }
 
+  dvd_demux->mpeg_version = mpeg_version;
   return str;
 }
 
@@ -508,26 +544,35 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
     guint8 stream_nr, gint type, const gpointer info)
 {
   GstDVDDemux *dvd_demux = GST_DVD_DEMUX (mpeg_demux);
-  guint8 sample_info = 0;
+  guint32 sample_info = 0;
   GstMPEGStream *str;
   GstDVDLPCMStream *lpcm_str = NULL;
-  GstCaps *caps;
-  gint width, rate, channels;
   gboolean add_pad = FALSE;
+  GstCaps *caps;
+  const gchar *codec = NULL, *lang_code = NULL;
 
   g_return_val_if_fail (stream_nr < GST_MPEG_DEMUX_NUM_AUDIO_STREAMS, NULL);
   g_return_val_if_fail (type > GST_MPEG_DEMUX_AUDIO_UNKNOWN &&
       type < GST_DVD_DEMUX_AUDIO_LAST, NULL);
 
   if (type < GST_MPEG_DEMUX_AUDIO_LAST) {
+    /* FIXME: language codes on MPEG audio streams */
     return parent_class->get_audio_stream (mpeg_demux, stream_nr, type, info);
   }
 
   if (type == GST_DVD_DEMUX_AUDIO_LPCM) {
-    sample_info = *((guint8 *) info);
+    sample_info = *((guint32 *) info);
   }
 
   str = mpeg_demux->audio_stream[stream_nr];
+
+  /* If the stream type is changing, recreate the pad */
+  if (str && str->type != type) {
+    gst_element_remove_pad (GST_ELEMENT (mpeg_demux), str->pad);
+    g_free (str);
+    str = mpeg_demux->audio_stream[stream_nr] = NULL;
+  }
+
   if (str == NULL) {
     gchar *name;
 
@@ -561,15 +606,21 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
   if (type != str->type ||
       (type == GST_DVD_DEMUX_AUDIO_LPCM &&
           sample_info != lpcm_str->sample_info)) {
+    gint width, rate, channels, dynamic_range;
+    gboolean emphasis, mute;
+
     /* We need to set new caps for this pad. */
     switch (type) {
       case GST_DVD_DEMUX_AUDIO_LPCM:
+        /* Dynamic range in the lower byte */
+        dynamic_range = sample_info & 0xff;
+
         /* Determine the sample width. */
-        switch (sample_info & 0xC0) {
-          case 0x80:
+        switch (sample_info & 0xC000) {
+          case 0x8000:
             width = 24;
             break;
-          case 0x40:
+          case 0x4000:
             width = 20;
             break;
           default:
@@ -578,32 +629,44 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
         }
 
         /* Determine the rate. */
-        if (sample_info & 0x10) {
+        if (sample_info & 0x1000) {
           rate = 96000;
         } else {
           rate = 48000;
         }
 
+        mute = ((sample_info & 0x400000) != 0);
+        emphasis = ((sample_info & 0x800000) != 0);
+
         /* Determine the number of channels. */
-        channels = (sample_info & 0x7) + 1;
+        channels = ((sample_info >> 8) & 0x7) + 1;
 
-        caps = gst_caps_new_simple ("audio/x-raw-int",
-            "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-            "signed", G_TYPE_BOOLEAN, TRUE,
+        caps = gst_caps_new_simple ("audio/x-dvd-lpcm",
             "width", G_TYPE_INT, width,
-            "depth", G_TYPE_INT, width,
-            "rate", G_TYPE_INT, rate, "channels", G_TYPE_INT, channels, NULL);
-
+            "rate", G_TYPE_INT, rate,
+            "channels", G_TYPE_INT, channels,
+            "dynamic_range", G_TYPE_INT, dynamic_range,
+            "emphasis", G_TYPE_BOOLEAN, emphasis,
+            "mute", G_TYPE_BOOLEAN, mute, NULL);
 
         lpcm_str->sample_info = sample_info;
+        lpcm_str->width = width;
+        lpcm_str->rate = rate;
+        lpcm_str->channels = channels;
+        lpcm_str->dynamic_range = dynamic_range;
+        lpcm_str->mute = mute;
+        lpcm_str->emphasis = emphasis;
+        codec = "LPCM audio";
         break;
 
       case GST_DVD_DEMUX_AUDIO_AC3:
         caps = gst_caps_new_simple ("audio/x-ac3", NULL);
+        codec = "AC-3 audio";
         break;
 
       case GST_DVD_DEMUX_AUDIO_DTS:
         caps = gst_caps_new_simple ("audio/x-dts", NULL);
+        codec = "DTS audio";
         break;
 
       default:
@@ -611,15 +674,52 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
         break;
     }
 
-    gst_pad_set_explicit_caps (str->pad, caps);
+    if (!gst_pad_set_explicit_caps (str->pad, caps)) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+          CORE, NEGOTIATION, (NULL), ("failed to set caps on pad %s:%s",
+              gst_element_get_name (dvd_demux), gst_pad_get_name (str->pad)));
+    }
 
     if (str->number == dvd_demux->cur_audio_nr) {
       /* This is the current audio stream.  Use the same caps. */
-      gst_pad_set_explicit_caps (dvd_demux->cur_audio, gst_caps_copy (caps));
+      if (!gst_pad_set_explicit_caps (dvd_demux->cur_audio,
+              gst_caps_copy (caps))) {
+        GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+            CORE, NEGOTIATION, (NULL), ("failed to set caps on pad %s:%s",
+                gst_element_get_name (dvd_demux),
+                gst_pad_get_name (dvd_demux->cur_audio)));
+      }
     }
-    if (add_pad)
+
+    if (add_pad) {
+      if (dvd_demux->langcodes) {
+        gchar *t;
+
+        t = g_strdup_printf ("audio-%d-language", stream_nr);
+        lang_code =
+            gst_structure_get_string (dvd_demux->langcodes->event_data.
+            structure.structure, t);
+        g_free (t);
+      }
+
       gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
 
+      if (codec || lang_code) {
+        GstTagList *list = gst_tag_list_new ();
+
+        if (codec) {
+          gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+              GST_TAG_AUDIO_CODEC, codec, NULL);
+        }
+        if (lang_code) {
+          gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+              GST_TAG_LANGUAGE_CODE, lang_code, NULL);
+        }
+        gst_element_found_tags_for_pad (GST_ELEMENT (mpeg_demux),
+            str->pad, 0, list);
+      }
+    }
+
     str->type = type;
   }
 
@@ -636,6 +736,7 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
   gchar *name;
   GstCaps *caps;
   gboolean add_pad = FALSE;
+  const gchar *lang_code = NULL;
 
   g_return_val_if_fail (stream_nr < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS, NULL);
   g_return_val_if_fail (type > GST_DVD_DEMUX_SUBP_UNKNOWN &&
@@ -645,11 +746,11 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
 
   if (str == NULL) {
     str = g_new0 (GstMPEGStream, 1);
-    str->type = GST_DVD_DEMUX_SUBP_UNKNOWN;
 
     name = g_strdup_printf ("subpicture_%02d", stream_nr);
     DEMUX_CLASS (dvd_demux)->init_stream (mpeg_demux, type, str, stream_nr,
         name, CLASS (dvd_demux)->subpicture_template);
+    str->type = GST_DVD_DEMUX_SUBP_UNKNOWN;
     g_free (name);
     add_pad = TRUE;
 
@@ -663,23 +764,51 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
   if (str->type != GST_DVD_DEMUX_SUBP_DVD) {
     /* We need to set new caps for this pad. */
     caps = gst_caps_new_simple ("video/x-dvd-subpicture", NULL);
-    gst_pad_set_explicit_caps (str->pad, caps);
+
+    if (!gst_pad_set_explicit_caps (str->pad, caps)) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+          CORE, NEGOTIATION, (NULL), ("failed to set caps on pad %s:%s",
+              gst_element_get_name (dvd_demux), gst_pad_get_name (str->pad)));
+    }
 
     if (str->number == dvd_demux->cur_subpicture_nr) {
       /* This is the current subpicture stream.  Use the same caps. */
-      gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, caps);
+      if (!gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, caps)) {
+        GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+            CORE, NEGOTIATION, (NULL), ("failed to set caps on pad %s:%s",
+                gst_element_get_name (dvd_demux), gst_pad_get_name (str->pad)));
+      }
     }
 
     gst_caps_free (caps);
-    if (add_pad)
+    if (add_pad) {
       gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
+
+      if (dvd_demux->langcodes) {
+        gchar *t;
+
+        t = g_strdup_printf ("subtitle-%d-language", stream_nr);
+        lang_code =
+            gst_structure_get_string (dvd_demux->langcodes->event_data.
+            structure.structure, t);
+        g_free (t);
+
+        if (lang_code) {
+          GstTagList *list = gst_tag_list_new ();
+
+          gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+              GST_TAG_LANGUAGE_CODE, lang_code, NULL);
+          gst_element_found_tags_for_pad (GST_ELEMENT (mpeg_demux),
+              str->pad, 0, list);
+        }
+      }
+    }
     str->type = GST_DVD_DEMUX_SUBP_DVD;
   }
 
   return str;
 }
 
-
 static void
 gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
     GstBuffer * buffer,
@@ -687,9 +816,10 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
 {
   GstDVDDemux *dvd_demux = GST_DVD_DEMUX (mpeg_demux);
   guint8 *basebuf;
-  guint8 ps_id_code, lpcm_sample_info;
+  guint8 ps_id_code;
   GstMPEGStream *outstream = NULL;
   guint first_access = 0;
+  gint align = 1, len, off;
 
   basebuf = GST_BUFFER_DATA (buffer);
 
@@ -699,7 +829,6 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
   /* In the following, the "first access" refers to the location in a
      buffer the time stamp is associated to.  DVDs include this
      information explicitely. */
-
   switch (stream_nr) {
     case 0:
       /* Private stream 1. */
@@ -712,25 +841,47 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
 
         /* Determine the position of the "first access".  This
            should always be the beginning of an AC3 frame. */
-        first_access = *(basebuf + headerlen + 6) * 256 +
-            *(basebuf + headerlen + 7);
+        first_access = (basebuf[headerlen + 6] << 8) | basebuf[headerlen + 7];
+
+        headerlen += 4;
+        datalen -= 4;
+      } else if (ps_id_code >= 0x88 && ps_id_code <= 0x8f) {
+        GST_LOG_OBJECT (dvd_demux,
+            "we have an audio (DTS) packet, track %d", ps_id_code - 0x88);
+        outstream = DEMUX_CLASS (dvd_demux)->get_audio_stream (mpeg_demux,
+            ps_id_code - 0x88, GST_DVD_DEMUX_AUDIO_DTS, NULL);
+
+        /* Determine the position of the "first access".  This
+           should always be the beginning of a DTS frame. */
+        first_access = (basebuf[headerlen + 6] << 8) | basebuf[headerlen + 7];
 
         headerlen += 4;
         datalen -= 4;
       } else if (ps_id_code >= 0xA0 && ps_id_code <= 0xA7) {
+        GstDVDLPCMStream *lpcm_str;
+        guint32 lpcm_sample_info;
+
         GST_LOG_OBJECT (dvd_demux,
             "we have an audio (LPCM) packet, track %d", ps_id_code - 0xA0);
-        lpcm_sample_info = basebuf[headerlen + 9];
+
+        /* Compose the sample info from the LPCM header, masking out the frame_num */
+        lpcm_sample_info =
+            basebuf[headerlen + 10] | (basebuf[headerlen +
+                9] << 8) | ((basebuf[headerlen + 8] & 0xc0) << 16);
+
         outstream = DEMUX_CLASS (dvd_demux)->get_audio_stream (mpeg_demux,
             ps_id_code - 0xA0, GST_DVD_DEMUX_AUDIO_LPCM, &lpcm_sample_info);
+        lpcm_str = (GstDVDLPCMStream *) outstream;
 
         /* Determine the position of the "first access". */
-        first_access = *(basebuf + headerlen + 6) * 256 +
-            *(basebuf + headerlen + 7);
+        first_access = (basebuf[headerlen + 6] << 8) | basebuf[headerlen + 7];
 
         /* Get rid of the LPCM header. */
         headerlen += 7;
         datalen -= 7;
+
+        /* align by frame round up to nearest byte */
+        align = (lpcm_str->width * lpcm_str->channels + 7) / 8;
       } else if (ps_id_code >= 0x20 && ps_id_code <= 0x3F) {
         GST_LOG_OBJECT (dvd_demux,
             "we have a subpicture packet, track %d", ps_id_code - 0x20);
@@ -784,18 +935,31 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
        a means to associate a timestamp to the middle of a buffer, we
        send two separate buffers and put the timestamp in the second
        one. */
-    DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
-        buffer, GST_CLOCK_TIME_NONE, headerlen + 4, first_access - 1);
-    DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
-        buffer, timestamp,
-        headerlen + 3 + first_access, datalen - (first_access - 1));
+    off = headerlen + 4;
+    len = first_access - 1;
+    len -= len % align;
+    if (len > 0) {
+      DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
+          buffer, GST_CLOCK_TIME_NONE, off, len);
+    }
+    off += len;
+    len = datalen - len;
+    len -= len % align;
+    if (len > 0) {
+      DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
+          buffer, timestamp, off, len);
+    }
   } else {
-    DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
-        buffer, timestamp, headerlen + 4, datalen);
+    off = headerlen + 4;
+    len = datalen;
+    len -= len % align;
+    if (len > 0) {
+      DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
+          buffer, timestamp, off, len);
+    }
   }
 }
 
-
 static void
 gst_dvd_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
     GstMPEGStream * outstream, GstBuffer * buffer,
@@ -810,8 +974,12 @@ gst_dvd_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
      minimize the time interval between the discontinuity and the data
      buffers following it. */
   if (dvd_demux->discont_time != GST_CLOCK_TIME_NONE) {
+    if ((gint64) (dvd_demux->discont_time) < 0) {
+      GST_ERROR ("DVD Discont < 0! % " G_GINT64_FORMAT,
+          (gint64) dvd_demux->discont_time);
+    }
     PARSE_CLASS (mpeg_demux)->send_discont (mpeg_parse,
-        dvd_demux->discont_time - 200 * GST_MSECOND);
+        dvd_demux->discont_time);
     dvd_demux->discont_time = GST_CLOCK_TIME_NONE;
   }
 
@@ -864,7 +1032,6 @@ gst_dvd_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
   gst_buffer_unref (buffer);
 }
 
-
 static void
 gst_dvd_demux_set_cur_audio (GstDVDDemux * dvd_demux, gint stream_nr)
 {
@@ -886,19 +1053,17 @@ gst_dvd_demux_set_cur_audio (GstDVDDemux * dvd_demux, gint stream_nr)
   str = mpeg_demux->audio_stream[stream_nr];
   if (str != NULL) {
     /* (Re)set the caps in the "current" pad. */
-    caps = gst_pad_get_negotiated_caps (str->pad);
+    caps = GST_RPAD_EXPLICIT_CAPS (str->pad);
     if (caps != NULL) {
       gst_pad_set_explicit_caps (dvd_demux->cur_audio, caps);
     }
   }
 }
 
-
 static void
 gst_dvd_demux_set_cur_subpicture (GstDVDDemux * dvd_demux, gint stream_nr)
 {
   GstMPEGStream *str;
-  const GstCaps *caps = NULL;
 
   g_return_if_fail (stream_nr >= -1 &&
       stream_nr < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS);
@@ -914,14 +1079,132 @@ gst_dvd_demux_set_cur_subpicture (GstDVDDemux * dvd_demux, gint stream_nr)
 
   str = dvd_demux->subpicture_stream[stream_nr];
   if (str != NULL) {
+    GstCaps *caps = NULL;
+
     /* (Re)set the caps in the "current" pad. */
-    caps = gst_pad_get_negotiated_caps (str->pad);
-    if (caps != NULL) {
-      gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, caps);
+    caps = GST_RPAD_EXPLICIT_CAPS (str->pad);
+    gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, caps);
+  }
+}
+
+static void
+gst_dvd_demux_reset (GstDVDDemux * dvd_demux)
+{
+  int i;
+  GstMPEGDemux *mpeg_demux = GST_MPEG_DEMUX (dvd_demux);
+
+  GST_INFO ("Resetting the dvd demuxer");
+  for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
+    if (dvd_demux->subpicture_stream[i]) {
+      gst_element_remove_pad (GST_ELEMENT (dvd_demux),
+          dvd_demux->subpicture_stream[i]->pad);
+      g_free (dvd_demux->subpicture_stream[i]);
+      dvd_demux->subpicture_stream[i] = NULL;
+    }
+  }
+  gst_pad_set_explicit_caps (dvd_demux->cur_video, NULL);
+  gst_pad_set_explicit_caps (dvd_demux->cur_audio, NULL);
+  gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, NULL);
+
+  dvd_demux->cur_video_nr = 0;
+  dvd_demux->cur_audio_nr = 0;
+  dvd_demux->cur_subpicture_nr = 0;
+  dvd_demux->mpeg_version = 0;
+  dvd_demux->last_end_ptm = INITIAL_END_PTM;
+
+  dvd_demux->discont_time = GST_CLOCK_TIME_NONE;
+
+  /* Reset max_gap handling */
+  mpeg_demux->max_gap = GST_CLOCK_TIME_NONE;
+  mpeg_demux->max_gap_tolerance = GST_CLOCK_TIME_NONE;
+}
+
+static void
+gst_dvd_demux_synchronise_pads (GstMPEGDemux * mpeg_demux,
+    GstClockTime threshold, GstClockTime new_ts)
+{
+  GstDVDDemux *dvd_demux = GST_DVD_DEMUX (mpeg_demux);
+  int i;
+
+  parent_class->synchronise_pads (mpeg_demux, threshold, new_ts);
+
+  for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
+    if (dvd_demux->subpicture_stream[i]
+        && (dvd_demux->subpicture_stream[i]->cur_ts < threshold)) {
+      DEMUX_CLASS (mpeg_demux)->sync_stream_to_time (mpeg_demux,
+          dvd_demux->subpicture_stream[i], new_ts);
+      dvd_demux->subpicture_stream[i]->cur_ts = new_ts;
     }
   }
 }
 
+static void
+gst_dvd_demux_sync_stream_to_time (GstMPEGDemux * mpeg_demux,
+    GstMPEGStream * stream, GstClockTime last_ts)
+{
+  GstDVDDemux *dvd_demux = GST_DVD_DEMUX (mpeg_demux);
+  GstClockTime start_ts;
+  GstEvent *filler = NULL;
+  GstFormat fmt = GST_FORMAT_TIME;
+  GstPad *outpad = NULL;
+  gint cur_nr = 0;
+
+  parent_class->sync_stream_to_time (mpeg_demux, stream, last_ts);
+
+  switch (GST_MPEG_DEMUX_STREAM_KIND (stream->type)) {
+    case GST_MPEG_DEMUX_STREAM_VIDEO:
+      outpad = dvd_demux->cur_video;
+      cur_nr = dvd_demux->cur_video_nr;
+      break;
+    case GST_MPEG_DEMUX_STREAM_AUDIO:
+      outpad = dvd_demux->cur_audio;
+      cur_nr = dvd_demux->cur_audio_nr;
+      break;
+    case GST_DVD_DEMUX_STREAM_SUBPICTURE:
+      outpad = dvd_demux->cur_subpicture;
+      cur_nr = dvd_demux->cur_subpicture_nr;
+      break;
+  }
+
+  if ((outpad != NULL) && (cur_nr == stream->number)) {
+    if (GST_PAD_PEER (stream->pad)
+        && gst_pad_query (GST_PAD_PEER (stream->pad), GST_QUERY_POSITION, &fmt,
+            (gint64 *) & start_ts)) {
+      if (start_ts < last_ts)
+        filler =
+            gst_event_new_filler_stamped (start_ts, GST_CLOCK_DIFF (last_ts,
+                start_ts));
+    } else
+      filler = gst_event_new_filler_stamped (last_ts, GST_CLOCK_TIME_NONE);
+
+    if (filler) {
+      if (GST_PAD_IS_USABLE (stream->pad))
+        gst_pad_push (stream->pad, GST_DATA (filler));
+      else
+        gst_event_unref (filler);
+    }
+  }
+}
+
+static GstElementStateReturn
+gst_dvd_demux_change_state (GstElement * element)
+{
+  GstDVDDemux *dvd_demux = GST_DVD_DEMUX (element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_PAUSED_TO_READY:
+      gst_dvd_demux_reset (dvd_demux);
+      if (dvd_demux->langcodes) {
+        gst_event_unref (dvd_demux->langcodes);
+        dvd_demux->langcodes = NULL;
+      }
+      break;
+    default:
+      break;
+  }
+
+  return GST_ELEMENT_CLASS (parent_class)->change_state (element);
+}
 
 gboolean
 gst_dvd_demux_plugin_init (GstPlugin * plugin)
