@@ -29,6 +29,7 @@
 GST_DEBUG_CATEGORY_STATIC (gst_play_base_bin_debug);
 #define GST_CAT_DEFAULT gst_play_base_bin_debug
 
+#define DEFAULT_QUEUE_THRESHOLD (2 * GST_SECOND)
 #define DEFAULT_QUEUE_SIZE  (3 * GST_SECOND)
 
 /* props */
@@ -37,9 +38,9 @@ enum
   ARG_0,
   ARG_URI,
   ARG_SUBURI,
-  ARG_THREADED,
-  ARG_NSTREAMS,
   ARG_QUEUE_SIZE,
+  ARG_QUEUE_THRESHOLD,
+  ARG_NSTREAMS,
   ARG_STREAMINFO,
   ARG_SOURCE,
   ARG_VIDEO,
@@ -56,6 +57,7 @@ enum
   GROUP_SWITCH_SIGNAL,
   LINK_STREAM_SIGNAL,
   UNLINK_STREAM_SIGNAL,
+  REDIRECT,
   LAST_SIGNAL
 };
 
@@ -82,6 +84,7 @@ static void gst_play_base_bin_error (GstElement * element,
 static void gst_play_base_bin_found_tag (GstElement * element,
     GstElement * source, const GstTagList * taglist, gpointer data);
 
+static gboolean prepare_output (GstPlayBaseBin * play_base_bin);
 static void set_active_source (GstPlayBaseBin * play_base_bin,
     GstStreamType type, gint source_num);
 static gboolean probe_triggered (GstProbe * probe, GstData ** data,
@@ -141,13 +144,19 @@ gst_play_base_bin_class_init (GstPlayBaseBinClass * klass)
   g_object_class_install_property (gobject_klass, ARG_SUBURI,
       g_param_spec_string ("suburi", ".sub-URI", "Optional URI of a subtitle",
           NULL, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_klass, ARG_NSTREAMS,
-      g_param_spec_int ("nstreams", "NStreams", "number of streams",
-          0, G_MAXINT, 0, G_PARAM_READABLE));
+
   g_object_class_install_property (gobject_klass, ARG_QUEUE_SIZE,
       g_param_spec_uint64 ("queue-size", "Queue size",
           "Size of internal queues in nanoseconds", 0, G_MAXINT64,
           DEFAULT_QUEUE_SIZE, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_klass, ARG_QUEUE_THRESHOLD,
+      g_param_spec_uint64 ("queue-threshold", "Queue threshold",
+          "Buffering threshold of internal queues in nanoseconds", 0,
+          G_MAXINT64, DEFAULT_QUEUE_THRESHOLD, G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_klass, ARG_NSTREAMS,
+      g_param_spec_int ("nstreams", "NStreams", "number of streams",
+          0, G_MAXINT, 0, G_PARAM_READABLE));
   g_object_class_install_property (gobject_klass, ARG_STREAMINFO,
       g_param_spec_pointer ("stream-info", "Stream info", "List of streaminfo",
           G_PARAM_READABLE));
@@ -192,6 +201,12 @@ gst_play_base_bin_class_init (GstPlayBaseBinClass * klass)
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstPlayBaseBinClass, group_switch),
       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+  gst_play_base_bin_signals[REDIRECT] =
+      g_signal_new ("got-redirect", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST,
+      G_STRUCT_OFFSET (GstPlayBaseBinClass, got_redirect),
+      NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1,
+      G_TYPE_STRING);
 
   /* action signals */
   gst_play_base_bin_signals[LINK_STREAM_SIGNAL] =
@@ -227,6 +242,7 @@ gst_play_base_bin_init (GstPlayBaseBin * play_base_bin)
   play_base_bin->uri = NULL;
   play_base_bin->suburi = NULL;
   play_base_bin->need_rebuild = TRUE;
+  play_base_bin->is_stream = FALSE;
   play_base_bin->source = NULL;
   play_base_bin->decoder = NULL;
   play_base_bin->subtitle = NULL;
@@ -238,6 +254,7 @@ gst_play_base_bin_init (GstPlayBaseBin * play_base_bin)
   play_base_bin->queued_groups = NULL;
 
   play_base_bin->queue_size = DEFAULT_QUEUE_SIZE;
+  play_base_bin->queue_threshold = DEFAULT_QUEUE_THRESHOLD;
 
   GST_FLAG_SET (play_base_bin, GST_BIN_SELF_SCHEDULABLE);
 }
@@ -371,7 +388,7 @@ group_destroy (GstPlayBaseGroup * group)
  * and ready for playback
  */
 static void
-group_commit (GstPlayBaseBin * play_base_bin, gboolean fatal)
+group_commit (GstPlayBaseBin * play_base_bin, gboolean fatal, gboolean subtitle)
 {
   GstPlayBaseGroup *group = play_base_bin->building_group;
   gboolean had_active_group = (get_active_group (play_base_bin) != NULL);
@@ -384,7 +401,7 @@ group_commit (GstPlayBaseBin * play_base_bin, gboolean fatal)
     } else {
       GST_DEBUG ("Group loading failed, bailing out");
     }
-  } else {
+  } else if (!subtitle) {
     gint n;
 
     GST_DEBUG ("group %p done", group);
@@ -417,7 +434,11 @@ group_commit (GstPlayBaseBin * play_base_bin, gboolean fatal)
   GST_DEBUG ("signaled group done");
   g_mutex_unlock (play_base_bin->group_lock);
 
-  if (!had_active_group && GST_STATE (play_base_bin) > GST_STATE_READY) {
+  if (!subtitle && play_base_bin->is_stream && !had_active_group &&
+      GST_STATE (play_base_bin) > GST_STATE_READY) {
+    if (!prepare_output (play_base_bin))
+      return;
+
     setup_substreams (play_base_bin);
     GST_DEBUG ("Emitting signal");
     g_signal_emit (play_base_bin,
@@ -441,6 +462,30 @@ group_is_muted (GstPlayBaseGroup * group)
   return TRUE;
 }
 
+/*
+ * Buffer/cache checking.
+ */
+
+static gboolean
+check_queue (GstProbe * probe, GstData ** data, gpointer user_data)
+{
+  GstElement *queue = GST_ELEMENT (user_data);
+  GstPlayBaseBin *play_base_bin = g_object_get_data (G_OBJECT (queue), "pbb");
+  guint64 level = 0;
+
+  g_object_get (G_OBJECT (queue), "current-level-time", &level, NULL);
+  GST_DEBUG ("Queue size: %" GST_TIME_FORMAT, GST_TIME_ARGS (level));
+  level = level * 100 / play_base_bin->queue_threshold;
+  if (level > 100)
+    level = 100;
+
+  g_signal_emit (play_base_bin,
+      gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, level);
+
+  /* continue! */
+  return TRUE;
+}
+
 /* this signal will be fired when one of the queues with raw
  * data is filled. This means that the group building stage is over 
  * and playback of the new queued group should start */
@@ -448,7 +493,72 @@ static void
 queue_overrun (GstElement * element, GstPlayBaseBin * play_base_bin)
 {
   GST_DEBUG ("queue %s overrun", gst_element_get_name (element));
-  group_commit (play_base_bin, FALSE);
+
+  group_commit (play_base_bin, FALSE, FALSE);
+
+  g_signal_handlers_disconnect_by_func (element,
+      G_CALLBACK (queue_overrun), play_base_bin);
+}
+
+/* Used for time-based buffering. */
+static void
+queue_threshold_reached (GstElement * queue, GstPlayBaseBin * play_base_bin)
+{
+  GstProbe *probe;
+
+  GST_DEBUG ("Running");
+
+  /* play */
+  g_object_set (queue, "min-threshold-time", (guint64) 0, NULL);
+
+  if ((probe = g_object_get_data (G_OBJECT (queue), "probe"))) {
+    GstPad *sinkpad;
+
+    sinkpad = gst_element_get_pad (queue, "sink");
+    GST_DEBUG_OBJECT (play_base_bin,
+        "Removing buffer probe %p from pad %s:%s (%p)",
+        probe, GST_DEBUG_PAD_NAME (sinkpad), sinkpad);
+
+    g_signal_emit (play_base_bin,
+        gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, 100);
+
+    g_object_set_data (G_OBJECT (queue), "probe", NULL);
+    gst_pad_remove_probe (sinkpad, probe);
+    gst_probe_destroy (probe);
+
+    g_signal_emit (play_base_bin,
+        gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, 100);
+  }
+}
+
+static void
+queue_out_of_data (GstElement * queue, GstPlayBaseBin * play_base_bin)
+{
+  GstProbe *probe;
+
+  GST_DEBUG ("Underrun, re-caching");
+
+  /* On underrun, we want to temoprarily pause playback, set a "min-size"
+   * threshold and wait for the running signal and then play again. Take
+   * care of possible deadlocks and so on, */
+  g_object_set (queue, "min-threshold-time",
+      (guint64) play_base_bin->queue_threshold, NULL);
+
+  /* re-connect probe */
+  if (!(probe = g_object_get_data (G_OBJECT (queue), "probe"))) {
+    GstPad *sinkpad;
+
+    probe = gst_probe_new (FALSE, check_queue, queue);
+    sinkpad = gst_element_get_pad (queue, "sink");
+    gst_pad_add_probe (sinkpad, probe);
+    g_object_set_data (G_OBJECT (queue), "probe", probe);
+    GST_DEBUG_OBJECT (play_base_bin,
+        "Re-attaching buffering probe %p to pad %s:%s (%p)",
+        probe, GST_DEBUG_PAD_NAME (sinkpad), sinkpad);
+
+    g_signal_emit (play_base_bin,
+        gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, 0);
+  }
 }
 
 /* generate a preroll element which is simply a queue. While there
@@ -491,6 +601,31 @@ gen_preroll_element (GstPlayBaseBin * play_base_bin,
       "max-size-time", play_base_bin->queue_size, NULL);
   sig = g_signal_connect (G_OBJECT (preroll), "overrun",
       G_CALLBACK (queue_overrun), play_base_bin);
+  if (play_base_bin->is_stream &&
+      ((type == GST_STREAM_TYPE_VIDEO &&
+              group->type[GST_STREAM_TYPE_AUDIO - 1].npads == 0) ||
+          (type == GST_STREAM_TYPE_AUDIO &&
+              group->type[GST_STREAM_TYPE_VIDEO - 1].npads == 0))) {
+    GstProbe *probe;
+    GstPad *sinkpad;
+
+    g_signal_connect (G_OBJECT (preroll), "running",
+        G_CALLBACK (queue_threshold_reached), play_base_bin);
+    g_object_set (G_OBJECT (preroll),
+        "min-threshold-time", (guint64) play_base_bin->queue_threshold, NULL);
+
+    /* give updates on queue size */
+    probe = gst_probe_new (FALSE, check_queue, preroll);
+    sinkpad = gst_element_get_pad (preroll, "sink");
+    gst_pad_add_probe (sinkpad, probe);
+    GST_DEBUG_OBJECT (play_base_bin, "Attaching probe %p to pad %s:%s (%p)",
+        probe, GST_DEBUG_PAD_NAME (sinkpad), sinkpad);
+    g_object_set_data (G_OBJECT (preroll), "pbb", play_base_bin);
+    g_object_set_data (G_OBJECT (preroll), "probe", probe);
+
+    g_signal_connect (G_OBJECT (preroll), "underrun",
+        G_CALLBACK (queue_out_of_data), play_base_bin);
+  }
   /* keep a ref to the signal id so that we can disconnect the signal callback
    * when we are done with the preroll */
   g_object_set_data (G_OBJECT (preroll), "signal_id", GINT_TO_POINTER (sig));
@@ -583,7 +718,7 @@ unknown_type (GstElement * element, GstPad * pad, GstCaps * caps,
   GstPlayBaseGroup *group = get_building_group (play_base_bin);
 
   capsstr = gst_caps_to_string (caps);
-  g_message /*warning */ ("don't know how to handle %s", capsstr);
+  g_message ("don't know how to handle %s", capsstr);
 
   /* add the stream to the list */
   info = gst_stream_info_new (GST_OBJECT (pad), GST_STREAM_TYPE_UNKNOWN,
@@ -622,7 +757,9 @@ no_more_pads (GstElement * element, GstPlayBaseBin * play_base_bin)
   /* setup phase */
   GST_DEBUG ("no more pads");
   /* we can commit this group for playback now */
-  group_commit (play_base_bin, FALSE);
+  group_commit (play_base_bin, play_base_bin->is_stream,
+      gst_object_get_parent (GST_OBJECT (element)) ==
+      GST_OBJECT (play_base_bin->subtitle));
 }
 
 static gboolean
@@ -674,12 +811,12 @@ probe_triggered (GstProbe * probe, GstData ** data, gpointer user_data)
         gst_element_set_state (play_base_bin->thread, GST_STATE_PAUSED);
         /* ok, get rid of the current group then */
         group_destroy (group);
-        g_signal_emit (play_base_bin,
-            gst_play_base_bin_signals[GROUP_SWITCH_SIGNAL], 0);
         /* removing the current group brings the next group
          * active */
         play_base_bin->queued_groups =
             g_list_remove (play_base_bin->queued_groups, group);
+        g_signal_emit (play_base_bin,
+            gst_play_base_bin_signals[GROUP_SWITCH_SIGNAL], 0);
         if (play_base_bin->queued_groups) {
           setup_substreams (play_base_bin);
           GST_DEBUG ("switching to next group %p",
@@ -819,9 +956,13 @@ new_decoded_pad (GstElement * element, GstPad * pad, gboolean last,
 
   group->nstreams++;
 
-  if (g_str_has_prefix (mimetype, "audio/")) {
+  if (g_str_has_prefix (mimetype, "audio/") &&
+      gst_object_get_parent (GST_OBJECT (element)) !=
+      GST_OBJECT (play_base_bin->subtitle)) {
     type = GST_STREAM_TYPE_AUDIO;
-  } else if (g_str_has_prefix (mimetype, "video/")) {
+  } else if (g_str_has_prefix (mimetype, "video/") &&
+      gst_object_get_parent (GST_OBJECT (element)) !=
+      GST_OBJECT (play_base_bin->subtitle)) {
     type = GST_STREAM_TYPE_VIDEO;
   } else if (g_str_has_prefix (mimetype, "text/")) {
     type = GST_STREAM_TYPE_TEXT;
@@ -862,7 +1003,7 @@ new_decoded_pad (GstElement * element, GstPad * pad, gboolean last,
 
   /* signal the no more pads after adding the stream */
   if (last)
-    no_more_pads (NULL, play_base_bin);
+    no_more_pads (element, play_base_bin);
 }
 
 
@@ -901,106 +1042,12 @@ state_change (GstElement * element,
   if (old_state > new_state) {
     /* EOS or error occurred, we have to commit the current group */
     GST_DEBUG ("state changed downwards");
-    group_commit (play_base_bin, TRUE);
+    group_commit (play_base_bin, TRUE,
+        gst_object_get_parent (GST_OBJECT (element)) ==
+        GST_OBJECT (play_base_bin->subtitle));
   }
 }
 
-/*
- * Buffer/cache checking. FIXME: make configurable.
- * Note that we could also do this (buffering) at the
- * preroll-level. The advantage there is that it'd
- * allow us to cache in time-units rather than byte-
- * units. Ohwell...
- */
-
-static gboolean
-check_queue (GstProbe * probe, GstData ** data, gpointer user_data)
-{
-  GstElement *queue = GST_ELEMENT (user_data);
-  GstPlayBaseBin *play_base_bin = g_object_get_data (G_OBJECT (queue), "pbb");
-  guint level = 0;
-
-  g_object_get (G_OBJECT (queue), "current-level-bytes", &level, NULL);
-  GST_DEBUG ("Queue size: %u bytes", level);
-  level = level * 100 / (64 * 1024);
-  if (level > 100)
-    level = 100;
-
-  g_signal_emit (play_base_bin,
-      gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, level);
-
-  /* continue! */
-  return TRUE;
-}
-
-static void
-buffer_underrun (GstElement * queue, GstPlayBaseBin * play_base_bin)
-{
-  GstProbe *probe;
-  GstPad *sourcepad;
-
-  GST_DEBUG ("Underrun, re-caching");
-
-  /* On underrun, we want to temoprarily pause playback, set a "min-size"
-   * treshold and wait for the running signal and then play again. Take
-   * care of possible deadlocks and so on, */
-  g_object_set (queue, "min-threshold-bytes", 64 * 1024, NULL);
-
-  g_signal_emit (play_base_bin,
-      gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, 0);
-
-  /* re-connect probe */
-  if (!(probe = g_object_get_data (G_OBJECT (queue), "probe"))) {
-    probe = gst_probe_new (FALSE, check_queue, queue);
-    sourcepad = GST_PAD_PEER (gst_element_get_pad (queue, "sink"));
-    gst_pad_add_probe (sourcepad, probe);
-    g_object_set_data (G_OBJECT (queue), "probe", probe);
-    GST_DEBUG_OBJECT (play_base_bin,
-        "Re-attaching buffering probe %p to pad %s:%s (%p)",
-        probe, GST_DEBUG_PAD_NAME (sourcepad), sourcepad);
-  }
-}
-
-static void
-buffer_running (GstElement * queue, GstPlayBaseBin * play_base_bin)
-{
-  GstProbe *probe;
-
-  GST_DEBUG ("Running");
-
-  /* When we had an underrun, we now want to play again. */
-  g_object_set (queue, "min-threshold-bytes", 0,
-      "max-size-bytes", 512 * 1024, NULL);
-
-  if ((probe = g_object_get_data (G_OBJECT (queue), "probe"))) {
-    GstPad *sourcepad;
-
-    sourcepad = GST_PAD_PEER (gst_element_get_pad (queue, "sink"));
-    GST_DEBUG_OBJECT (play_base_bin,
-        "Removing buffer probe %p from pad %s:%s (%p)",
-        probe, GST_DEBUG_PAD_NAME (sourcepad), sourcepad);
-
-    g_signal_emit (play_base_bin,
-        gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, 100);
-
-    g_object_set_data (G_OBJECT (queue), "probe", NULL);
-    gst_pad_remove_probe (sourcepad, probe);
-    gst_probe_destroy (probe);
-  }
-}
-
-static void
-buffer_overrun (GstElement * queue, GstPlayBaseBin * play_base_bin)
-{
-  GST_DEBUG ("Overrun, leaking upstream and flushing next few buffers");
-
-  /* we want to decrease max-size here so the next few bytes are flushed */
-  g_object_set (queue, "max-size-bytes", 448 * 1024, NULL);
-
-  /*g_signal_emit (play_base_bin,
-     gst_play_base_bin_signals[BUFFERING_SIGNAL], 0, 100); */
-}
-
 /*
  * Generate source ! subparse bins.
  */
@@ -1009,21 +1056,15 @@ static GstElement *
 setup_subtitle (GstPlayBaseBin * play_base_bin, gchar * sub_uri)
 {
   GstElement *source, *subparse, *subbin;
-  gchar *name;
 
   source = gst_element_make_from_uri (GST_URI_SRC, sub_uri, NULL);
   if (!source)
     return NULL;
+  subparse = gst_element_factory_make ("decodebin", "subtitle-decoder");
 
-  subparse = gst_element_factory_make ("subparse", NULL);
-  name = g_strdup_printf ("subbin");
-  subbin = gst_thread_new (name);
-  g_free (name);
-
+  subbin = gst_thread_new ("subbin");
   gst_bin_add_many (GST_BIN (subbin), source, subparse, NULL);
   gst_element_link (source, subparse);
-  gst_element_add_ghost_pad (subbin,
-      gst_element_get_pad (subparse, "src"), "src");
 
   /* return the subtitle GstElement object */
   return subbin;
@@ -1034,13 +1075,9 @@ setup_subtitle (GstPlayBaseBin * play_base_bin, gchar * sub_uri)
  */
 
 static GstElement *
-gen_source_element (GstPlayBaseBin * play_base_bin,
-    GstElement ** subbin, gboolean * _is_stream)
+gen_source_element (GstPlayBaseBin * play_base_bin, GstElement ** subbin)
 {
-  GstElement *source, *queue, *bin;
-  GstProbe *probe;
-  gboolean is_stream;
-  GstPad *sourcepad;
+  GstElement *source;
 
   /* stip subtitle from uri */
   if (!play_base_bin->uri)
@@ -1059,38 +1096,12 @@ gen_source_element (GstPlayBaseBin * play_base_bin,
     return NULL;
 
   /* lame - FIXME, maybe we can use seek_types/mask here? */
-  *_is_stream = is_stream = !strncmp (play_base_bin->uri, "http://", 7) ||
-      !strncmp (play_base_bin->uri, "mms://", 6);
-  if (!is_stream)
-    return source;
+  play_base_bin->is_stream = !strncmp (play_base_bin->uri, "http://", 7) ||
+      !strncmp (play_base_bin->uri, "mms://", 6) ||
+      !strncmp (play_base_bin->uri, "rtp://", 6) ||
+      !strncmp (play_base_bin->uri, "rtsp://", 7);
 
-  /* buffer */
-  bin = gst_thread_new ("sourcebin");
-  queue = gst_element_factory_make ("queue", "buffer");
-  g_object_set (queue, "max-size-bytes", 512 * 1024,
-      "max-size-buffers", 0, NULL);
-  /* I'd like it to be leaky too, but only for live sources. How? */
-  g_signal_connect (queue, "underrun", G_CALLBACK (buffer_underrun),
-      play_base_bin);
-  g_signal_connect (queue, "running", G_CALLBACK (buffer_running),
-      play_base_bin);
-  g_signal_connect (queue, "overrun", G_CALLBACK (buffer_overrun),
-      play_base_bin);
-
-  /* give updates on queue size */
-  probe = gst_probe_new (FALSE, check_queue, queue);
-  sourcepad = gst_element_get_pad (source, "src");
-  gst_pad_add_probe (sourcepad, probe);
-  GST_DEBUG_OBJECT (play_base_bin, "Attaching probe %p to pad %s:%s (%p)",
-      probe, GST_DEBUG_PAD_NAME (sourcepad), sourcepad);
-  g_object_set_data (G_OBJECT (queue), "pbb", play_base_bin);
-  g_object_set_data (G_OBJECT (queue), "probe", probe);
-
-  gst_element_link (source, queue);
-  gst_bin_add_many (GST_BIN (bin), source, queue, NULL);
-  gst_element_add_ghost_pad (bin, gst_element_get_pad (queue, "src"), "src");
-
-  return bin;
+  return source;
 }
 
 /* Setup the substreams (to be called right after group_commit ()) */
@@ -1137,18 +1148,29 @@ setup_substreams (GstPlayBaseBin * play_base_bin)
   }
 }
 
+/*
+ * Called when we're redirected to a new URI.
+ */
+static void
+got_redirect (GstElement * element, const gchar * new_location, gpointer data)
+{
+  gchar **location = data;
+
+  if (!*location)
+    *location = g_strdup (new_location);
+}
+
 /* construct and run the source and decoder elements until we found
  * all the streams or until a preroll queue has been filled.
- */
+*/
 static gboolean
 setup_source (GstPlayBaseBin * play_base_bin,
-    gboolean * _stream, GError ** error)
+    gchar ** new_location, GError ** error)
 {
   GstElement *old_src;
   GstElement *old_dec;
   GstPad *srcpad = NULL;
-  GstElement *subbin;
-  gboolean stream;
+  GstElement *subbin = NULL;
 
   if (!play_base_bin->need_rebuild)
     return TRUE;
@@ -1159,8 +1181,7 @@ setup_source (GstPlayBaseBin * play_base_bin,
   old_src = play_base_bin->source;
 
   /* create and configure an element that can handle the uri */
-  play_base_bin->source = gen_source_element (play_base_bin, &subbin, &stream);
-  *_stream = stream;
+  play_base_bin->source = gen_source_element (play_base_bin, &subbin);
 
   if (!play_base_bin->source) {
     /* whoops, could not create the source element */
@@ -1199,13 +1220,69 @@ setup_source (GstPlayBaseBin * play_base_bin,
 
   /* do subs */
   if (subbin) {
+    gint sig4, sig5, sig6;
+    GstElement *db;
+
     play_base_bin->subtitle = subbin;
+    db = gst_bin_get_by_name (GST_BIN (subbin), "subtitle-decoder");
 
     /* don't add yet, because we will preroll, and subs shouldn't
      * preroll (we shouldn't preroll more than once source). */
     gst_element_set_state (subbin, GST_STATE_PAUSED);
-    new_decoded_pad (subbin, gst_element_get_pad (subbin, "src"), FALSE,
-        play_base_bin);
+
+    /* do type detection, without adding (so no preroll) */
+    g_signal_connect (G_OBJECT (db),
+        "new-decoded-pad", G_CALLBACK (new_decoded_pad), play_base_bin);
+    g_signal_connect (G_OBJECT (db), "no-more-pads",
+        G_CALLBACK (no_more_pads), play_base_bin);
+
+    if (!play_base_bin->is_stream) {
+      sig4 = g_signal_connect (G_OBJECT (db),
+          "unknown-type", G_CALLBACK (unknown_type), play_base_bin);
+      sig5 = g_signal_connect (G_OBJECT (subbin), "error",
+          G_CALLBACK (thread_error), error);
+
+      /* either when the queues are filled or when the decoder element
+       * has no more dynamic streams, the cond is unlocked. We can remove
+       * the signal handlers then
+       */
+      g_mutex_lock (play_base_bin->group_lock);
+      if (gst_element_set_state (subbin, GST_STATE_PLAYING) ==
+          GST_STATE_SUCCESS) {
+        GST_DEBUG ("waiting for first group...");
+        sig6 = g_signal_connect (G_OBJECT (subbin),
+            "state-change", G_CALLBACK (state_change), play_base_bin);
+        g_cond_wait (play_base_bin->group_cond, play_base_bin->group_lock);
+        GST_DEBUG ("group done !");
+      } else {
+        GST_DEBUG ("state change failed, subtitle cannot be loaded");
+        sig6 = 0;
+      }
+      g_mutex_unlock (play_base_bin->group_lock);
+
+      if (sig6 != 0)
+        g_signal_handler_disconnect (G_OBJECT (subbin), sig6);
+
+      g_signal_handler_disconnect (G_OBJECT (subbin), sig5);
+      g_signal_handler_disconnect (G_OBJECT (db), sig4);
+
+      gst_element_set_state (subbin, GST_STATE_PAUSED);
+
+      if (!play_base_bin->building_group ||
+          play_base_bin->building_group->type[GST_STREAM_TYPE_TEXT - 1].npads ==
+          0) {
+        if (*error) {
+          g_error_free (*error);
+          *error = NULL;
+        }
+
+        GST_DEBUG ("No subtitle found - ignoring");
+        gst_object_unref (GST_OBJECT (play_base_bin->subtitle));
+        play_base_bin->subtitle = NULL;
+      } else {
+        GST_DEBUG ("Subtitle set-up successful");
+      }
+    }
   }
 
   /* now see if the source element emits raw audio/video all by itself,
@@ -1286,6 +1363,8 @@ setup_source (GstPlayBaseBin * play_base_bin,
         gst_object_unref (GST_OBJECT (old_dec));
     }
 
+    g_signal_connect (play_base_bin->decoder, "got_redirect",
+        G_CALLBACK (got_redirect), new_location);
     res = gst_pad_link (srcpad,
         gst_element_get_pad (play_base_bin->decoder, "sink"));
     if (!res) {
@@ -1299,7 +1378,7 @@ setup_source (GstPlayBaseBin * play_base_bin,
     sig3 = g_signal_connect (G_OBJECT (play_base_bin->decoder), "no-more-pads",
         G_CALLBACK (no_more_pads), play_base_bin);
 
-    if (!stream) {
+    if (!play_base_bin->is_stream) {
       sig4 = g_signal_connect (G_OBJECT (play_base_bin->decoder),
           "unknown-type", G_CALLBACK (unknown_type), play_base_bin);
       sig5 = g_signal_connect (G_OBJECT (play_base_bin->thread), "error",
@@ -1340,7 +1419,7 @@ setup_source (GstPlayBaseBin * play_base_bin,
       /* make subs iterate from now on */
       gst_bin_add (GST_BIN (play_base_bin->thread), play_base_bin->subtitle);
     }
-    if (!stream) {
+    if (!play_base_bin->is_stream) {
       setup_substreams (play_base_bin);
     }
   }
@@ -1348,6 +1427,59 @@ setup_source (GstPlayBaseBin * play_base_bin,
   return TRUE;
 }
 
+static gboolean
+prepare_output (GstPlayBaseBin * play_base_bin)
+{
+  const GList *item;
+  gboolean stream_found = FALSE, no_media = FALSE;
+  GstPlayBaseGroup *group;
+
+  group = get_active_group (play_base_bin);
+
+  /* check if we found any supported stream... if not, then
+   * we detected stream type (or the above would've failed),
+   * but linking/decoding failed - plugin probably missing. */
+  for (item = group ? group->streaminfo : NULL; item != NULL; item = item->next) {
+    GstStreamInfo *info = GST_STREAM_INFO (item->data);
+
+    if (info->type != GST_STREAM_TYPE_UNKNOWN) {
+      stream_found = TRUE;
+      break;
+    } else if (!item->prev && !item->next) {
+      /* We're no audio/video and the only stream... We could
+       * be something not-media that's detected because then our
+       * typefind doesn't mess up with mp3 (bz2, gz, elf, ...) */
+      if (info->caps && !gst_caps_is_empty (info->caps)) {
+        const gchar *mime =
+            gst_structure_get_name (gst_caps_get_structure (info->caps, 0));
+
+        if (!strcmp (mime, "application/x-executable") ||
+            !strcmp (mime, "application/x-bzip") ||
+            !strcmp (mime, "application/x-gzip") ||
+            !strcmp (mime, "application/zip") ||
+            !strcmp (mime, "application/x-compress")) {
+          no_media = TRUE;
+        }
+      }
+    }
+  }
+
+  if (!stream_found) {
+    if (!no_media) {
+      GST_ELEMENT_ERROR (play_base_bin, STREAM, CODEC_NOT_FOUND,
+          ("There were no decoders found to handle the stream in file "
+              "\"%s\", you might need to install the corresponding plugins",
+              play_base_bin->uri), (NULL));
+    } else {
+      GST_ELEMENT_ERROR (play_base_bin, STREAM, WRONG_TYPE,
+          ("File \"%s\" is not a media file", play_base_bin->uri), (NULL));
+    }
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 /*
  * Multi-stream management. -1 = none.
  */
@@ -1503,6 +1635,9 @@ gst_play_base_bin_set_property (GObject * object, guint prop_id,
     case ARG_QUEUE_SIZE:
       play_base_bin->queue_size = g_value_get_uint64 (value);
       break;
+    case ARG_QUEUE_THRESHOLD:
+      play_base_bin->queue_threshold = g_value_get_uint64 (value);
+      break;
     case ARG_VIDEO:
       set_active_source (play_base_bin,
           GST_STREAM_TYPE_VIDEO, g_value_get_int (value));
@@ -1552,6 +1687,9 @@ gst_play_base_bin_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_QUEUE_SIZE:
       g_value_set_uint64 (value, play_base_bin->queue_size);
       break;
+    case ARG_QUEUE_THRESHOLD:
+      g_value_set_uint64 (value, play_base_bin->queue_threshold);
+      break;
     case ARG_STREAMINFO:
       g_value_set_pointer (value,
           (gpointer) gst_play_base_bin_get_streaminfo (play_base_bin));
@@ -1627,9 +1765,9 @@ gst_play_base_bin_change_state (GstElement * element)
     case GST_STATE_READY_TO_PAUSED:
     {
       GError *error = NULL;
-      gboolean stream;
+      gchar *new_location = NULL;
 
-      if (!setup_source (play_base_bin, &stream, &error) || error != NULL) {
+      if (!setup_source (play_base_bin, &new_location, &error) || error != NULL) {
         if (!error) {
           /* opening failed but no error - hellup */
           GST_ELEMENT_ERROR (GST_ELEMENT (play_base_bin), STREAM,
@@ -1642,61 +1780,20 @@ gst_play_base_bin_change_state (GstElement * element)
           g_error_free (error);
         }
         ret = GST_STATE_FAILURE;
-      } else if (stream) {
+      } else if (new_location) {
+        g_signal_emit (play_base_bin, gst_play_base_bin_signals[REDIRECT],
+            0, new_location);
+        g_free (new_location);
+        ret = GST_STATE_FAILURE;
+      } else if (play_base_bin->is_stream) {
         ret = gst_element_set_state (play_base_bin->thread, GST_STATE_PAUSED);
+      } else if (!prepare_output (play_base_bin)) {
+        gst_element_set_state (play_base_bin->thread, GST_STATE_READY);
+        ret = GST_STATE_FAILURE;
       } else {
-        const GList *item;
-        gboolean stream_found = FALSE, no_media = FALSE;
-        GstPlayBaseGroup *group;
-
-        group = get_active_group (play_base_bin);
-
-        /* check if we found any supported stream... if not, then
-         * we detected stream type (or the above would've failed),
-         * but linking/decoding failed - plugin probably missing. */
-        for (item = group ? group->streaminfo : NULL;
-            item != NULL; item = item->next) {
-          GstStreamInfo *info = GST_STREAM_INFO (item->data);
-
-          if (info->type != GST_STREAM_TYPE_UNKNOWN) {
-            stream_found = TRUE;
-            break;
-          } else if (!item->prev && !item->next) {
-            /* We're no audio/video and the only stream... We could
-             * be something not-media that's detected because then our
-             * typefind doesn't mess up with mp3 (bz2, gz, elf, ...) */
-            if (info->caps && !gst_caps_is_empty (info->caps)) {
-              const gchar *mime =
-                  gst_structure_get_name (gst_caps_get_structure (info->caps,
-                      0));
-
-              if (!strcmp (mime, "application/x-executable") ||
-                  !strcmp (mime, "application/x-bzip") ||
-                  !strcmp (mime, "application/x-gzip") ||
-                  !strcmp (mime, "application/zip") ||
-                  !strcmp (mime, "application/x-compress")) {
-                no_media = TRUE;
-              }
-            }
-          }
-        }
-
-        if (!stream_found) {
-          if (!no_media) {
-            GST_ELEMENT_ERROR (play_base_bin, STREAM, CODEC_NOT_FOUND,
-                ("There were no decoders found to handle the stream in file "
-                    "\"%s\", you might need to install the corresponding plugins",
-                    play_base_bin->uri), (NULL));
-          } else {
-            GST_ELEMENT_ERROR (play_base_bin, STREAM, WRONG_TYPE,
-                ("File \"%s\" is not a media file", play_base_bin->uri),
-                (NULL));
-          }
-          ret = GST_STATE_FAILURE;
-        } else {
-          ret = gst_element_set_state (play_base_bin->thread, GST_STATE_PAUSED);
-        }
+        ret = gst_element_set_state (play_base_bin->thread, GST_STATE_PAUSED);
       }
+
       if (ret == GST_STATE_SUCCESS) {
         /* error forwarding:
          * we only connect after the stream has been set up. If that failed,
@@ -1706,7 +1803,7 @@ gst_play_base_bin_change_state (GstElement * element)
             G_CALLBACK (gst_play_base_bin_error), play_base_bin);
         g_signal_connect (G_OBJECT (play_base_bin->thread), "eos",
             G_CALLBACK (play_base_eos), play_base_bin);
-        if (!stream) {
+        if (!play_base_bin->is_stream) {
           GST_DEBUG ("emit signal");
           g_signal_emit (play_base_bin,
               gst_play_base_bin_signals[SETUP_OUTPUT_PADS_SIGNAL], 0);
@@ -1715,6 +1812,7 @@ gst_play_base_bin_change_state (GstElement * element)
       } else {
         /* clean up leftover groups */
         remove_groups (play_base_bin);
+        play_base_bin->need_rebuild = TRUE;
       }
       break;
     }
