@@ -120,6 +120,7 @@ static GstElementDetails gst_qtdemux_details = {
 
 enum
 {
+  SIGNAL_REDIRECT,
   LAST_SIGNAL
 };
 
@@ -147,6 +148,7 @@ GST_STATIC_PAD_TEMPLATE ("video_%02d",
     GST_PAD_SOMETIMES,
     GST_STATIC_CAPS_ANY);
 
+static guint qt_signals[LAST_SIGNAL] = { 0 };
 static GstElementClass *parent_class = NULL;
 
 static void gst_qtdemux_class_init (GstQTDemuxClass * klass);
@@ -173,9 +175,9 @@ static void qtdemux_tag_add_gnre (GstQTDemux * qtdemux, const char *tag,
 static void gst_qtdemux_handle_esds (GstQTDemux * qtdemux,
     QtDemuxStream * stream, GNode * esds);
 static GstCaps *qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
-    const guint8 * stsd_data);
+    const guint8 * stsd_data, const gchar ** codec_name);
 static GstCaps *qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc,
-    const guint8 * data, int len);
+    const guint8 * data, int len, const gchar ** codec_name);
 
 static GType
 gst_qtdemux_get_type (void)
@@ -224,6 +226,12 @@ gst_qtdemux_class_init (GstQTDemuxClass * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  qt_signals[SIGNAL_REDIRECT] = g_signal_new ("got-redirect",
+      G_TYPE_FROM_CLASS (gobject_class), G_SIGNAL_RUN_LAST,
+      G_STRUCT_OFFSET (GstQTDemuxClass, got_redirect),
+      NULL, NULL, g_cclosure_marshal_VOID__STRING,
+      G_TYPE_NONE, 1, G_TYPE_STRING);
+
   gstelement_class->change_state = gst_qtdemux_change_state;
 }
 
@@ -490,6 +498,9 @@ gst_qtdemux_handle_sink_event (GstQTDemux * qtdemux)
   GST_DEBUG ("qtdemux: event %p %d", event, type);
 
   switch (type) {
+    case GST_EVENT_INTERRUPT:
+      gst_event_unref (event);
+      return FALSE;
     case GST_EVENT_EOS:
       gst_bytestream_flush (qtdemux->bs, remaining);
       gst_pad_event_default (qtdemux->sinkpad, event);
@@ -572,7 +583,7 @@ gst_qtdemux_loop_header (GstElement * element)
   //cur_offset = gst_bytestream_tell(qtdemux->bs);
 
   cur_offset = qtdemux->offset;
-  GST_DEBUG ("loop at position %d", cur_offset);
+  GST_DEBUG ("loop at position %d, state %d", cur_offset, qtdemux->state);
 
   switch (qtdemux->state) {
     case QTDEMUX_STATE_HEADER:
@@ -617,14 +628,14 @@ gst_qtdemux_loop_header (GstElement * element)
         case GST_MAKE_FOURCC ('w', 'i', 'd', 'e'):
         case GST_MAKE_FOURCC ('P', 'I', 'C', 'T'):
         case GST_MAKE_FOURCC ('p', 'n', 'o', 't'):
-          break;
+          goto the_d;
         case GST_MAKE_FOURCC ('m', 'o', 'o', 'v'):
         {
           GstBuffer *moov;
 
           do {
             /* read in the complete data from the moov atom */
-            ret = gst_bytestream_read (qtdemux->bs, &moov, length);
+            ret = gst_bytestream_peek (qtdemux->bs, &moov, length);
             if (ret < length) {
               GST_DEBUG ("read failed (%d < %d)", ret, length);
               if (!gst_qtdemux_handle_sink_event (qtdemux)) {
@@ -634,6 +645,7 @@ gst_qtdemux_loop_header (GstElement * element)
               break;
             }
           } while (1);
+          gst_bytestream_flush_fast (qtdemux->bs, length);
           qtdemux->offset += length;
 
           qtdemux_parse_moov (qtdemux, GST_BUFFER_DATA (moov), length);
@@ -647,13 +659,26 @@ gst_qtdemux_loop_header (GstElement * element)
           qtdemux->state = QTDEMUX_STATE_MOVIE;
           break;
         }
+        the_d:
         default:
         {
+          guint32 pending;
+
           GST_LOG ("unknown %08x '" GST_FOURCC_FORMAT "' at %d",
               fourcc, GST_FOURCC_ARGS (fourcc), cur_offset);
+          gst_bytestream_get_status (qtdemux->bs, &pending, NULL);
+          if (length <= pending)
+            gst_bytestream_flush_fast (qtdemux->bs, length);
+          else {
+            offset = cur_offset + length;
+            if (!gst_bytestream_seek (qtdemux->bs, offset, GST_SEEK_METHOD_SET)) {
+              gst_bytestream_flush (qtdemux->bs, length);
+            }
+          }
           break;
         }
       }
+#if 0
       ret = gst_bytestream_seek (qtdemux->bs, cur_offset + length,
           GST_SEEK_METHOD_SET);
       GST_DEBUG ("seek returned %d", ret);
@@ -667,6 +692,7 @@ gst_qtdemux_loop_header (GstElement * element)
           }
         }
       }
+#endif
       qtdemux->offset = cur_offset + length;
       break;
     }
@@ -677,8 +703,10 @@ gst_qtdemux_loop_header (GstElement * element)
       do {
         ret = gst_bytestream_peek_bytes (qtdemux->bs, &data, 1);
         if (ret < 1) {
-          if (!gst_qtdemux_handle_sink_event (qtdemux)) {
+          if (gst_qtdemux_handle_sink_event (qtdemux)) {
             return;
+          } else {
+            break;
           }
         } else {
           break;
@@ -750,8 +778,11 @@ gst_qtdemux_loop_header (GstElement * element)
               return;
             }
           }
-        } else if (ret == FALSE && offset < cur_offset)
-          GST_ERROR ("cannot flush backwards");
+        } else if (ret == FALSE && offset < cur_offset) {
+          GST_ERROR ("cannot flush backwards, %d < %d", offset, cur_offset);
+          stream->sample_index++;
+          return;
+        }
         qtdemux->offset = offset;
         return;
       }
@@ -759,7 +790,7 @@ gst_qtdemux_loop_header (GstElement * element)
       GST_DEBUG ("reading %d bytes", size);
       buf = NULL;
       do {
-        ret = gst_bytestream_read (qtdemux->bs, &buf, size);
+        ret = gst_bytestream_peek (qtdemux->bs, &buf, size);
         if (ret < size) {
           GST_DEBUG ("read failed (%d < %d)", ret, size);
           if (!gst_qtdemux_handle_sink_event (qtdemux)) {
@@ -769,6 +800,7 @@ gst_qtdemux_loop_header (GstElement * element)
           break;
         }
       } while (TRUE);
+      gst_bytestream_flush_fast (qtdemux->bs, size);
       qtdemux->offset += size;
 
       if (buf) {
@@ -829,7 +861,8 @@ gst_qtdemux_loop_header (GstElement * element)
 }
 
 void
-gst_qtdemux_add_stream (GstQTDemux * qtdemux, QtDemuxStream * stream)
+gst_qtdemux_add_stream (GstQTDemux * qtdemux,
+    QtDemuxStream * stream, GstTagList * list)
 {
   gchar *caps;
 
@@ -886,6 +919,10 @@ gst_qtdemux_add_stream (GstQTDemux * qtdemux, QtDemuxStream * stream)
   GST_DEBUG ("adding pad %s %p to qtdemux %p",
       gst_pad_get_name (stream->pad), stream->pad, qtdemux);
   gst_element_add_pad (GST_ELEMENT (qtdemux), stream->pad);
+  if (list) {
+    gst_element_found_tags_for_pad (GST_ELEMENT (qtdemux),
+        stream->pad, 0, list);
+  }
 }
 
 
@@ -957,7 +994,9 @@ gst_qtdemux_add_stream (GstQTDemux * qtdemux, QtDemuxStream * stream)
 #define FOURCC_free	GST_MAKE_FOURCC('f','r','e','e')
 #define FOURCC_data	GST_MAKE_FOURCC('d','a','t','a')
 #define FOURCC_SVQ3	GST_MAKE_FOURCC('S','V','Q','3')
-
+#define FOURCC_rmra	GST_MAKE_FOURCC('r','m','r','a')
+#define FOURCC_rmda	GST_MAKE_FOURCC('r','m','d','a')
+#define FOURCC_rdrf	GST_MAKE_FOURCC('r','d','r','f')
 
 static void qtdemux_dump_mvhd (GstQTDemux * qtdemux, void *buffer, int depth);
 static void qtdemux_dump_tkhd (GstQTDemux * qtdemux, void *buffer, int depth);
@@ -1058,6 +1097,9 @@ QtNodeType qt_node_types[] = {
   {FOURCC_data, "data", 0, qtdemux_dump_unknown},
   {FOURCC_free, "free", 0,},
   {FOURCC_SVQ3, "SVQ3", 0,},
+  {FOURCC_rmra, "rmra", QT_CONTAINER,},
+  {FOURCC_rmda, "rmda", QT_CONTAINER,},
+  {FOURCC_rdrf, "rdrf", 0,},
   {0, "unknown", 0},
 };
 static int n_qt_node_types = sizeof (qt_node_types) / sizeof (qt_node_types[0]);
@@ -1240,9 +1282,10 @@ qtdemux_parse (GstQTDemux * qtdemux, GNode * node, void *buffer, int length)
       guint32 version;
 
       version = QTDEMUX_GUINT32_GET (buffer + 16);
-      if (version == 0x00010000) {
-        buf = buffer + 0x34;
+      if (version == 0x00010000 || 1) {
+        buf = buffer + 0x24;
         end = buffer + length;
+
         while (buf < end) {
           GNode *child;
 
@@ -1903,6 +1946,22 @@ qtdemux_parse_tree (GstQTDemux * qtdemux)
 
   mvhd = qtdemux_tree_get_child_by_type (qtdemux->moov_node, FOURCC_mvhd);
   if (mvhd == NULL) {
+    GNode *rmra, *rmda, *rdrf;
+
+    rmra = qtdemux_tree_get_child_by_type (qtdemux->moov_node, FOURCC_rmra);
+    if (rmra) {
+      rmda = qtdemux_tree_get_child_by_type (rmra, FOURCC_rmda);
+      if (rmra) {
+        rdrf = qtdemux_tree_get_child_by_type (rmda, FOURCC_rdrf);
+        if (rdrf) {
+          GST_LOG ("New location: %s", (char *) rdrf->data + 20);
+          g_signal_emit (qtdemux, qt_signals[SIGNAL_REDIRECT], 0,
+              (char *) rdrf->data + 20);
+          return;
+        }
+      }
+    }
+
     GST_LOG ("No mvhd node found.");
     return;
   }
@@ -1975,6 +2034,8 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
   guint64 timestamp;
   int sample_size;
   int sample_index;
+  GstTagList *list = NULL;
+  const gchar *codec = NULL;
 
   stream = g_new0 (QtDemuxStream, 1);
 
@@ -2000,7 +2061,8 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
    * some of those trailers, nowadays, have prologue images that are
    * themselves vide tracks as well. I haven't really found a way to
    * identify those yet, except for just looking at their duration. */
-  if ((guint64) QTDEMUX_GUINT32_GET (mdhd->data + 24) *
+  if (stream->timescale * qtdemux->duration != 0 &&
+      (guint64) QTDEMUX_GUINT32_GET (mdhd->data + 24) *
       qtdemux->timescale * 10 / (stream->timescale * qtdemux->duration) < 2) {
     GST_WARNING ("Track shorter than 20%% (%d/%d vs. %d/%d) of the stream "
         "found, assuming preview image or something; skipping track",
@@ -2043,7 +2105,12 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
         QTDEMUX_GUINT16_GET (stsd->data + offset + 48));
 
     stream->fourcc = fourcc = QTDEMUX_FOURCC_GET (stsd->data + offset + 4);
-    stream->caps = qtdemux_video_caps (qtdemux, fourcc, stsd->data);
+    stream->caps = qtdemux_video_caps (qtdemux, fourcc, stsd->data, &codec);
+    if (codec) {
+      list = gst_tag_list_new ();
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+          GST_TAG_VIDEO_CODEC, codec, NULL);
+    }
 
     esds = NULL;
     mp4v = qtdemux_tree_get_child_by_type (stsd, FOURCC_mp4v);
@@ -2133,15 +2200,24 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
       GST_ERROR ("unknown version %08x", version);
     }
 
-    stream->caps = qtdemux_audio_caps (qtdemux, fourcc, NULL, 0);
+    stream->caps = qtdemux_audio_caps (qtdemux, fourcc, NULL, 0, &codec);
+    if (codec) {
+      list = gst_tag_list_new ();
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+          GST_TAG_AUDIO_CODEC, codec, NULL);
+    }
 
     mp4a = qtdemux_tree_get_child_by_type (stsd, FOURCC_mp4a);
     wave = NULL;
     if (mp4a)
       wave = qtdemux_tree_get_child_by_type (mp4a, FOURCC_wave);
+
     esds = NULL;
     if (wave)
       esds = qtdemux_tree_get_child_by_type (wave, FOURCC_esds);
+    else if (mp4a)
+      esds = qtdemux_tree_get_child_by_type (mp4a, FOURCC_esds);
+
     if (esds) {
       gst_qtdemux_handle_esds (qtdemux, stream, esds);
 #if 0
@@ -2154,6 +2230,38 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
       gst_caps_set_simple (stream->caps, "codec_data",
           GST_TYPE_BUFFER, buffer, NULL);
 #endif
+    } else {
+      if (QTDEMUX_FOURCC_GET (stsd->data + 16 + 4) ==
+          GST_MAKE_FOURCC ('Q', 'D', 'M', '2')) {
+        gint len = QTDEMUX_GUINT32_GET (stsd->data);
+
+        if (len > 0x4C) {
+          GstBuffer *buf = gst_buffer_new_and_alloc (len - 0x4C);
+
+          memcpy (GST_BUFFER_DATA (buf),
+              (guint8 *) stsd->data + 0x4C, len - 0x4C);
+          gst_caps_set_simple (stream->caps,
+              "codec_data", GST_TYPE_BUFFER, buf, NULL);
+          gst_buffer_unref (buf);
+        }
+        gst_caps_set_simple (stream->caps,
+            "samplesize", G_TYPE_INT, samplesize, NULL);
+      } else if (QTDEMUX_FOURCC_GET (stsd->data + 16 + 4) ==
+          GST_MAKE_FOURCC ('a', 'l', 'a', 'c')) {
+        gint len = QTDEMUX_GUINT32_GET (stsd->data);
+
+        if (len > 0x34) {
+          GstBuffer *buf = gst_buffer_new_and_alloc (len - 0x34);
+
+          memcpy (GST_BUFFER_DATA (buf),
+              (guint8 *) stsd->data + 0x34, len - 0x34);
+          gst_caps_set_simple (stream->caps,
+              "codec_data", GST_TYPE_BUFFER, buf, NULL);
+          gst_buffer_unref (buf);
+        }
+        gst_caps_set_simple (stream->caps,
+            "samplesize", G_TYPE_INT, samplesize, NULL);
+      }
     }
     GST_INFO ("type " GST_FOURCC_FORMAT " caps %" GST_PTR_FORMAT,
         GST_FOURCC_ARGS (QTDEMUX_FOURCC_GET (stsd->data + 16 + 4)),
@@ -2344,7 +2452,7 @@ done2:
       break;
   }
 #endif
-  gst_qtdemux_add_stream (qtdemux, stream);
+  gst_qtdemux_add_stream (qtdemux, stream, list);
 }
 
 static void
@@ -2595,21 +2703,29 @@ gst_qtdemux_handle_esds (GstQTDemux * qtdemux, QtDemuxStream * stream,
   }
 }
 
+#define _codec(name) \
+  do { \
+    if (codec_name) { \
+      *codec_name = name; \
+    } \
+  } while (0)
+
 static GstCaps *
 qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
-    const guint8 * stsd_data)
+    const guint8 * stsd_data, const gchar ** codec_name)
 {
   switch (fourcc) {
     case GST_MAKE_FOURCC ('j', 'p', 'e', 'g'):
-      /* JPEG */
+      _codec ("JPEG still images");
       return gst_caps_from_string ("image/jpeg");
     case GST_MAKE_FOURCC ('m', 'j', 'p', 'a'):
-      /* Motion-JPEG (format A) */
+      _codec ("Motion-JPEG");
       return gst_caps_from_string ("image/jpeg");
     case GST_MAKE_FOURCC ('m', 'j', 'p', 'b'):
-      /* Motion-JPEG (format B) */
+      _codec ("Motion-JPEG format B");
       return gst_caps_from_string ("image/jpeg-b");
     case GST_MAKE_FOURCC ('S', 'V', 'Q', '3'):
+      _codec ("Sorensen video v.3");
       if (stsd_data != NULL) {
         gst_getbits_t gb;
         gint halfpel_flag;
@@ -2662,9 +2778,10 @@ qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
       return gst_caps_from_string ("video/x-svq, " "svqversion = (int) 3");
     case GST_MAKE_FOURCC ('s', 'v', 'q', 'i'):
     case GST_MAKE_FOURCC ('S', 'V', 'Q', '1'):
+      _codec ("Sorensen video v.1");
       return gst_caps_from_string ("video/x-svq, " "svqversion = (int) 1");
     case GST_MAKE_FOURCC ('r', 'a', 'w', ' '):
-      /* uncompressed RGB */
+      _codec ("Raw RGB video");
       return gst_caps_from_string ("video/x-raw-rgb, "
           "endianness = (int) BIG_ENDIAN");
       /*"bpp", GST_PROPS_INT(x),
@@ -2673,35 +2790,37 @@ qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
          "green_mask", GST_PROPS_INT(x),
          "blue_mask", GST_PROPS_INT(x), FIXME! */
     case GST_MAKE_FOURCC ('Y', 'u', 'v', '2'):
-      /* uncompressed YUV2 */
+      _codec ("Raw packed YUV 4:2:2");
       return gst_caps_from_string ("video/x-raw-yuv, "
           "format = (fourcc) YUY2");
     case GST_MAKE_FOURCC ('m', 'p', 'e', 'g'):
-      /* MPEG */
+      _codec ("MPEG-1 video");
       return gst_caps_from_string ("video/mpeg, "
           "systemstream = (boolean) false, " "mpegversion = (int) 1");
     case GST_MAKE_FOURCC ('g', 'i', 'f', ' '):
+      _codec ("GIF still images");
       return gst_caps_from_string ("image/gif");
     case GST_MAKE_FOURCC ('h', '2', '6', '3'):
     case GST_MAKE_FOURCC ('s', '2', '6', '3'):
-      /* H.263 */
+      _codec ("H.263");
       /* ffmpeg uses the height/width props, don't know why */
       return gst_caps_from_string ("video/x-h263");
     case GST_MAKE_FOURCC ('m', 'p', '4', 'v'):
-      /* MPEG-4 */
+      _codec ("MPEG-4 video");
       return gst_caps_from_string ("video/mpeg, "
           "mpegversion = (int) 4, " "systemstream = (boolean) false");
     case GST_MAKE_FOURCC ('3', 'I', 'V', '1'):
     case GST_MAKE_FOURCC ('3', 'I', 'V', '2'):
+      _codec ("3ivX video");
       return gst_caps_from_string ("video/x-3ivx");
     case GST_MAKE_FOURCC ('c', 'v', 'i', 'd'):
-      /* Cinepak */
+      _codec ("Cinepak");
       return gst_caps_from_string ("video/x-cinepak");
     case GST_MAKE_FOURCC ('r', 'p', 'z', 'a'):
-      /* Apple Video */
+      _codec ("Apple video");
       return gst_caps_from_string ("video/x-apple-video");
     case GST_MAKE_FOURCC ('a', 'v', 'c', '1'):
-      /* H.264/AVC */
+      _codec ("H.264 / AVC");
       return gst_caps_from_string ("video/x-h264");
     case GST_MAKE_FOURCC ('r', 'l', 'e', ' '):
       /* Run-length encoding */
@@ -2725,7 +2844,7 @@ qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
 
 static GstCaps *
 qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data,
-    int len)
+    int len, const gchar ** codec_name)
 {
   switch (fourcc) {
 #if 0
@@ -2733,50 +2852,61 @@ qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data,
       return NULL;              /*gst_caps_from_string ("audio/raw"); */
 #endif
     case GST_MAKE_FOURCC ('r', 'a', 'w', ' '):
+      _codec ("Raw 8-bit PCM audio");
       /* FIXME */
       return gst_caps_from_string ("audio/x-raw-int, "
           "width = (int) 8, " "depth = (int) 8, " "signed = (boolean) true");
     case GST_MAKE_FOURCC ('t', 'w', 'o', 's'):
+      _codec ("Raw 16-bit PCM audio");
       /* FIXME */
       return gst_caps_from_string ("audio/x-raw-int, "
           "width = (int) 16, "
           "depth = (int) 16, "
           "endianness = (int) BIG_ENDIAN, " "signed = (boolean) true");
     case GST_MAKE_FOURCC ('s', 'o', 'w', 't'):
+      _codec ("Raw 16-bit PCM audio");
       /* FIXME */
       return gst_caps_from_string ("audio/x-raw-int, "
           "width = (int) 16, "
           "depth = (int) 16, "
           "endianness = (int) G_LITTLE_ENDIAN, " "signed = (boolean) true");
     case GST_MAKE_FOURCC ('f', 'l', '6', '4'):
+      _codec ("Raw 64-bit floating-point audio");
       return gst_caps_from_string ("audio/x-raw-float, "
           "width = (int) 64, " "endianness = (int) G_BIG_ENDIAN");
     case GST_MAKE_FOURCC ('f', 'l', '3', '2'):
+      _codec ("Raw 32-bit floating-point audio");
       return gst_caps_from_string ("audio/x-raw-float, "
           "width = (int) 32, " "endianness = (int) G_BIG_ENDIAN");
     case GST_MAKE_FOURCC ('i', 'n', '2', '4'):
+      _codec ("Raw 24-bit PCM audio");
       /* FIXME */
       return gst_caps_from_string ("audio/x-raw-int, "
           "width = (int) 24, "
           "depth = (int) 32, "
           "endianness = (int) G_BIG_ENDIAN, " "signed = (boolean) true");
     case GST_MAKE_FOURCC ('i', 'n', '3', '2'):
+      _codec ("Raw 32-bit PCM audio");
       /* FIXME */
       return gst_caps_from_string ("audio/x-raw-int, "
           "width = (int) 32, "
           "depth = (int) 32, "
           "endianness = (int) G_BIG_ENDIAN, " "signed = (boolean) true");
     case GST_MAKE_FOURCC ('u', 'l', 'a', 'w'):
+      _codec ("Mu-law audio");
       /* FIXME */
       return gst_caps_from_string ("audio/x-mulaw");
     case GST_MAKE_FOURCC ('a', 'l', 'a', 'w'):
+      _codec ("A-law audio");
       /* FIXME */
       return gst_caps_from_string ("audio/x-alaw");
     case 0x6d730002:
+      _codec ("Microsoft ADPCM");
       /* Microsoft ADPCM-ACM code 2 */
       return gst_caps_from_string ("audio/x-adpcm, "
           "layout = (string) microsoft");
     case 0x6d730011:
+      _codec ("DVI/Intel IMA ADPCM");
       /* FIXME DVI/Intel IMA ADPCM/ACM code 17 */
       return gst_caps_from_string ("audio/x-adpcm, "
           "layout = (string) quicktime");
@@ -2784,26 +2914,28 @@ qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data,
       /* MPEG layer 3, CBR only (pre QT4.1) */
     case 0x5500736d:
     case GST_MAKE_FOURCC ('.', 'm', 'p', '3'):
+      _codec ("MPEG-1 layer 3");
       /* MPEG layer 3, CBR & VBR (QT4.1 and later) */
       return gst_caps_from_string ("audio/mpeg, "
           "layer = (int) 3, " "mpegversion = (int) 1");
     case GST_MAKE_FOURCC ('M', 'A', 'C', '3'):
-      /* MACE 3:1 */
+      _codec ("MACE-3");
       return gst_caps_from_string ("audio/x-mace, " "maceversion = (int) 3");
     case GST_MAKE_FOURCC ('M', 'A', 'C', '6'):
-      /* MACE 6:1 */
+      _codec ("MACE-6");
       return gst_caps_from_string ("audio/x-mace, " "maceversion = (int) 6");
     case GST_MAKE_FOURCC ('O', 'g', 'g', 'V'):
-      /* Ogg Vorbis */
+      /* ogg/vorbis */
       return gst_caps_from_string ("application/ogg");
     case GST_MAKE_FOURCC ('d', 'v', 'c', 'a'):
-      /* DV audio */
+      _codec ("DV audio");
       return gst_caps_from_string ("audio/x-dv");
     case GST_MAKE_FOURCC ('m', 'p', '4', 'a'):
-      /* MPEG-4 AAC */
+      _codec ("MPEG-4 AAC audio");
       return gst_caps_new_simple ("audio/mpeg",
-          "mpegversion", G_TYPE_INT, 4, NULL);
+          "mpegversion", G_TYPE_INT, 4, "framed", G_TYPE_BOOLEAN, TRUE, NULL);
     case GST_MAKE_FOURCC ('Q', 'D', 'M', '2'):
+      _codec ("QDesign Music v.2");
       /* FIXME: QDesign music version 2 (no constant) */
       if (data) {
         return gst_caps_new_simple ("audio/x-qdm2",
@@ -2814,15 +2946,18 @@ qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data,
         return gst_caps_new_simple ("audio/x-qdm2", NULL);
       }
     case GST_MAKE_FOURCC ('a', 'g', 's', 'm'):
-      /* GSM */
+      _codec ("GSM audio");
       return gst_caps_new_simple ("audio/x-gsm", NULL);
     case GST_MAKE_FOURCC ('s', 'a', 'm', 'r'):
-      /* AMR-NB */
+      _codec ("AMR audio");
       return gst_caps_new_simple ("audio/x-amr-nb", NULL);
     case GST_MAKE_FOURCC ('i', 'm', 'a', '4'):
-      /* IMA 4:1 */
+      _codec ("Quicktime IMA ADPCM");
       return gst_caps_new_simple ("audio/x-adpcm",
           "layout", G_TYPE_STRING, "quicktime", NULL);
+    case GST_MAKE_FOURCC ('a', 'l', 'a', 'c'):
+      _codec ("Apple lossless audio");
+      return gst_caps_new_simple ("audio/x-alac", NULL);
     case GST_MAKE_FOURCC ('q', 't', 'v', 'r'):
       /* ? */
     case GST_MAKE_FOURCC ('Q', 'D', 'M', 'C'):
