@@ -48,7 +48,8 @@ enum
   ARG_HOST,
   ARG_PORT,
   ARG_CONTROL,
-  ARG_MTU
+  ARG_MTU,
+  ARG_TTL
       /* FILL ME */
 };
 
@@ -63,6 +64,7 @@ gst_udpsink_control_get_type (void)
     {CONTROL_TCP, "3", "tcp"},
     {CONTROL_ZERO, NULL, NULL},
   };
+
   if (!udpsink_control_type) {
     udpsink_control_type =
         g_enum_register_static ("GstUDPSinkControl", udpsink_control);
@@ -107,6 +109,7 @@ gst_udpsink_get_type (void)
       (GInstanceInitFunc) gst_udpsink_init,
       NULL
     };
+
     udpsink_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstUDPSink", &udpsink_info,
         0);
@@ -143,7 +146,8 @@ gst_udpsink_class_init (GstUDPSink * klass)
   g_object_class_install_property (gobject_class, ARG_CONTROL,
       g_param_spec_enum ("control", "control", "The type of control",
           GST_TYPE_UDPSINK_CONTROL, CONTROL_UDP, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, ARG_MTU, g_param_spec_int ("mtu", "mtu", "maximun transmit unit", G_MININT, G_MAXINT, 0, G_PARAM_READWRITE)); /* CHECKME */
+  g_object_class_install_property (gobject_class, ARG_MTU, g_param_spec_int ("mtu", "mtu", "maximum transmit unit", G_MININT, G_MAXINT, 0, G_PARAM_READWRITE)); /* CHECKME */
+  g_object_class_install_property (gobject_class, ARG_TTL, g_param_spec_int ("ttl", "ttl", "time to live", G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));  /* CHECKME */
 
   gobject_class->set_property = gst_udpsink_set_property;
   gobject_class->get_property = gst_udpsink_get_property;
@@ -156,6 +160,7 @@ gst_udpsink_class_init (GstUDPSink * klass)
 static GstPadLinkReturn
 gst_udpsink_sink_link (GstPad * pad, const GstCaps * caps)
 {
+#ifndef GST_DISABLE_LOADSAVE
   GstUDPSink *udpsink;
   struct sockaddr_in serv_addr;
   struct hostent *serverhost;
@@ -163,7 +168,6 @@ gst_udpsink_sink_link (GstPad * pad, const GstCaps * caps)
   FILE *f;
   guint bc_val;
 
-#ifndef GST_DISABLE_LOADSAVE
   xmlDocPtr doc;
   xmlChar *buf;
   int buf_size;
@@ -184,8 +188,9 @@ gst_udpsink_sink_link (GstPad * pad, const GstCaps * caps)
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_port = htons (udpsink->port + 1);
 
-  doc = xmlNewDoc ("1.0");
-  doc->xmlRootNode = xmlNewDocNode (doc, NULL, "NewCaps", NULL);
+  doc = xmlNewDoc ((const xmlChar *) "1.0");
+  doc->xmlRootNode =
+      xmlNewDocNode (doc, NULL, (const xmlChar *) "NewCaps", NULL);
 
   gst_caps_save_thyself (caps, doc->xmlRootNode);
 
@@ -262,6 +267,7 @@ gst_udpsink_init (GstUDPSink * udpsink)
   udpsink->port = UDP_DEFAULT_PORT;
   udpsink->control = CONTROL_UDP;
   udpsink->mtu = 1024;
+  udpsink->ttl = 1;
 
   udpsink->clock = NULL;
 }
@@ -340,6 +346,9 @@ gst_udpsink_set_property (GObject * object, guint prop_id, const GValue * value,
     case ARG_MTU:
       udpsink->mtu = g_value_get_int (value);
       break;
+    case ARG_TTL:
+      udpsink->ttl = g_value_get_int (value);
+      break;
     default:
       break;
   }
@@ -368,6 +377,9 @@ gst_udpsink_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_MTU:
       g_value_set_int (value, udpsink->mtu);
       break;
+    case ARG_TTL:
+      g_value_set_int (value, udpsink->ttl);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -389,6 +401,8 @@ gst_udpsink_init_send (GstUDPSink * sink)
 
   /* if its an IP address */
   if (inet_aton (sink->host, &addr)) {
+    int ttl = htons (sink->ttl);
+
     /* check if its a multicast address */
     if ((ntohl (addr.s_addr) & 0xe0000000) == 0xe0000000) {
       sink->multi_addr.imr_multiaddr.s_addr = addr.s_addr;
@@ -399,6 +413,8 @@ gst_udpsink_init_send (GstUDPSink * sink)
       /* Joining the multicast group */
       setsockopt (sink->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, &sink->multi_addr,
           sizeof (sink->multi_addr));
+      /* set the time to live */
+      setsockopt (sink->sock, IPPROTO_IP, IP_TTL, &ttl, sizeof (ttl));
     }
 
     else {
