@@ -240,6 +240,7 @@ gst_videoscale_link (GstPad * pad, const GstCaps * caps)
   if (GST_PAD_LINK_SUCCESSFUL (ret)) {
     /* cool, we can use passthru */
     GST_DEBUG_OBJECT (videoscale, "passthru works");
+    GST_FLAG_SET (videoscale, GST_ELEMENT_WORK_IN_PLACE);
 
     videoscale->passthru = TRUE;
     newwidth = width;
@@ -248,6 +249,8 @@ gst_videoscale_link (GstPad * pad, const GstCaps * caps)
     goto beach;
   }
 
+  GST_FLAG_UNSET (videoscale, GST_ELEMENT_WORK_IN_PLACE);
+
   /* no passthru, so try to convert */
   GST_DEBUG_OBJECT (videoscale, "no passthru");
 
@@ -486,29 +489,22 @@ gst_videoscale_handle_src_event (GstPad * pad, GstEvent * event)
   GstVideoscale *videoscale;
   double a;
   GstStructure *structure;
-  GstEvent *new_event;
 
   videoscale = GST_VIDEOSCALE (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NAVIGATION:
-      structure = gst_structure_copy (event->event_data.structure.structure);
-      if (gst_structure_get_double (event->event_data.structure.structure,
-              "pointer_x", &a)) {
+      event = GST_EVENT (gst_data_copy_on_write (GST_DATA (event)));
+      structure = event->event_data.structure.structure;
+      if (gst_structure_get_double (structure, "pointer_x", &a)) {
         gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE,
             a * videoscale->from_width / videoscale->to_width, NULL);
       }
-      if (gst_structure_get_double (event->event_data.structure.structure,
-              "pointer_y", &a)) {
+      if (gst_structure_get_double (structure, "pointer_y", &a)) {
         gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE,
             a * videoscale->from_height / videoscale->to_height, NULL);
       }
-      gst_event_unref (event);
-      new_event = gst_event_new (GST_EVENT_NAVIGATION);
-      GST_DEBUG_OBJECT (videoscale, "creating new NAVIGATION event %p",
-          new_event);
-      new_event->event_data.structure.structure = structure;
-      return gst_pad_event_default (pad, new_event);
+      return gst_pad_event_default (pad, event);
       break;
     default:
       GST_DEBUG_OBJECT (videoscale, "passing on non-NAVIGATION event %p",
